/****************************************************************************
**
** Copyright (C) 2010-2011 Nokia Corporation and/or its subsidiary(-ies).
** All rights reserved.
** Contact: Nokia Corporation (ivan.frade@nokia.com)
**
** This file is part of QtSparqlTracker library.
**
** $QT_BEGIN_LICENSE:LGPL$
** Commercial Usage
** Licensees holding valid Qt Commercial licenses may use this file in
** accordance with the Qt Commercial License Agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and Nokia.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 2.1 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU Lesser General Public License version 2.1 requirements
** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** In addition, as a special exception, Nokia gives you certain additional
** rights.  These rights are described in the Nokia Qt LGPL Exception
** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3.0 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU General Public License version 3.0 requirements will be
** met: http://www.gnu.org/copyleft/gpl.html.
**
** If you have questions regarding the use of this file, please contact
** Nokia at ivan.frade@nokia.com.
** $QT_END_LICENSE$
**
****************************************************************************/

#include <QtGui/QApplication>
#include <QtDeclarative>
#include "TrackerLiveQuery"
#include <QtSparql>
#include <QDebug>

int main(int argc, char *argv[])
{
    QApplication app(argc, argv);

    QString origQuery("select tracker:id(?e) ?s "
                       "{ ?e a nmo:Email ; "
                       "nmo:messageSubject ?s . "
                       "} "
                       "order by ?e");
    QString updateQuery("select tracker:id(?e) ?s "
                         "{ ?e a nmo:Email ; "
                         "nmo:messageSubject ?s . "
                         "%FILTER "
                         "} "
                         "order by ?e");
    QSparqlConnection conn("QTRACKER_DIRECT");

    // Parameters: original query, number of columns, identity columns, and the
    // QSparqlConnection to run the queries with.

    // Identity columns are the set of columns which identify a row in the
    // model.
    TrackerLiveQuery liveQuery(origQuery, 2, QList<int>() << 0, conn);
    // Collation columns define the columns which are used in sorting, and the
    // sorting type.
    liveQuery.setCollationColumns(QList<TrackerLiveQuery::CollationColumn>() <<
                                   TrackerLiveQuery::CollationColumn(1, QVariant::String));

    // Parameters: the update query with a "%FILTER" placeholder; the update
    // query needs to have the same sorting criteria than the original query.
    TrackerPartialUpdater up(updateQuery);
    // Parameters:
    // 1) the class to watch
    // 2) the interesting properties (empty list == all)
    // 3) filter snippet for constructing the filter statement which will
    // replace "%FILTER" in the update query; the snippet contains the "%LIST"
    // placholder
    // 4) TrackerPartialUpdater::Subject means that %LIST will be replaced
    // with a list of tracker:id's extracted from the "subject" part of the
    // triples (subject, predicate, object) contained in the Tracker
    // GraphUpdated signal.
    // 5) The rows in the model which are affected by the update query will
    // contain one of those id's on column 0.
    up.watchClass("nmo:Message", QStringList(),
                   "tracker:id(?e) in %LIST", TrackerPartialUpdater::Subject, 0);

    liveQuery.addUpdater(up);
    liveQuery.start();

    // Get the model to use from the livequery
    QAbstractItemModel* model = liveQuery.model();

    // Set the header data, in QML you will be able to access the data using these
    // as property names
    model->setHeaderData(0, Qt::Horizontal, "TrackerId");
    model->setHeaderData(1, Qt::Horizontal, "Subject");

    QDeclarativeView viewQml;
    QDeclarativeContext *ctxt = viewQml.rootContext();

    //now set the context property for the ListView model to the liveQuery model
    ctxt->setContextProperty("contactModel", model);
    viewQml.setSource(QUrl::fromLocalFile("qml/liveQml/main.qml"));
    viewQml.show();

    return app.exec();
}

/*

How to insert example data into tracker:

tracker-sparql -qu "insert {<email0> a nmo:Email ; nmo:messageSubject \"foo\" .}"
tracker-sparql -qu "delete {<email0> a rdfs:Resource . }"

*/

