
#include <TrackerLiveQuery>

#include <QSparqlConnection>

#include <QApplication>
#include <QTableView>

// TODO Add a filter by uri to this query as defined in the original source
#define GEOTAG_QUERY "SELECT ?_geolocation \
                            nco:country(?_postal_address) AS ?_country \
                            nco:locality(?_postal_address) AS ?_city \
                            nco:region(?_postal_address) AS ?_district \
                            tracker:id(?_geolocation) AS ?_id \
                     WHERE \
                     { \
                         ?_geolocation a slo:GeoLocation . \
                         ?_postal_address a nco:PostalAddress . \
                         ?_geolocation slo:postalAddress ?_postal_address . \
                     } \
                     GROUP BY ?_geolocation \
                     ORDER BY ?_id"

// TODO Add a filter by uri to this query as defined in the original source
#define GEOTAG_UPDATE_QUERY "SELECT ?_geolocation \
                            nco:country(?_postal_address) AS ?_country \
                            nco:locality(?_postal_address) AS ?_city \
                            nco:region(?_postal_address) AS ?_district \
                            tracker:id(?_geolocation) AS ?_id \
                     WHERE \
                     { \
                         ?_geolocation a slo:GeoLocation . \
                         ?_postal_address a nco:PostalAddress . \
                         ?_geolocation slo:postalAddress ?_postal_address . \
                         %FILTER \
                     } \
                     GROUP BY ?_geolocation \
                     ORDER BY ?_id"

int main(int argc, char *argv[])
{
    QApplication app(argc, argv);

    QSparqlConnection connection ("QTRACKER_DIRECT");

    TrackerLiveQuery liveGeoQuery (QString (GEOTAG_QUERY), 5, connection);

    liveGeoQuery.setIdentityColumns(QList<int>() << 4);
    liveGeoQuery.setCollationColumns(QList<TrackerLiveQuery::CollationColumn>() <<
                                     TrackerLiveQuery::CollationColumn(4, QVariant::Int));

    TrackerPartialUpdater up(QString (GEOTAG_UPDATE_QUERY));

    up.watchClass("slo:GeoLocation", QStringList(),
                  "tracker:id(?_geolocation) in %LIST", TrackerPartialUpdater::Subject, 4);

    liveGeoQuery.addUpdater (up);
    liveGeoQuery.start();

    QTableView* view = new QTableView();
    view->setModel(liveGeoQuery.model());

    view->show();
    return app.exec();
}
