
#include <TrackerLiveQuery>

#include <QSparqlConnection>

#include <QApplication>
#include <QTableView>

#define TAG_QUERY "SELECT tracker:id(?tag) \
                   ?tag \
                   ?label \
                   ( \
                       SELECT COUNT(?freq_uri) as ?freq WHERE { ?freq_uri nao:hasTag ?tag .} \
                   ) AS ?freq \
                   tracker:isDefaultTag(?tag) \
                   GROUP_CONCAT(?uri,',') \
                   WHERE \
                   { \
                       ?tag a nao:Tag . \
                       OPTIONAL { \
                           ?uri nao:hasTag ?tag . \
                       } \
                       ?tag nao:prefLabel ?label \
                   } \
                   GROUP BY ?tag \
                   ORDER BY ?label"


#define TAG_UPDATE_QUERY "SELECT tracker:id(?tag) \
                         ?tag \
                         ?label \
                         ( \
                             SELECT COUNT(?freq_uri) as ?freq WHERE { ?freq_uri nao:hasTag ?tag .} \
                         ) AS ?freq \
                         tracker:isDefaultTag(?tag) \
                         GROUP_CONCAT(?uri,',') \
                   WHERE \
                   { \
                       ?tag a nao:Tag . \
                       OPTIONAL { \
                           ?uri nao:hasTag ?tag . \
                       } \
                       ?tag nao:prefLabel ?label \
                       %FILTER \
                   } \
                   GROUP BY ?tag \
                   ORDER BY ?label"

int main(int argc, char *argv[])
{
    QApplication app(argc, argv);

    QSparqlConnection connection ("QTRACKER_DIRECT");

    TrackerLiveQuery liveTagQuery (QString (TAG_QUERY), 6, connection);

    liveTagQuery.setIdentityColumns(QList<int>() << 0);
    liveTagQuery.setCollationColumns(QList<TrackerLiveQuery::CollationColumn>() <<
                                     TrackerLiveQuery::CollationColumn(2, QVariant::String));

    TrackerPartialUpdater up(QString (TAG_UPDATE_QUERY));

    up.watchClass("nao:Tag", QStringList(),
                  "tracker:id(?tag) in %LIST", TrackerPartialUpdater::Subject, 0);

    liveTagQuery.addUpdater (up);
    liveTagQuery.start();

    QTableView* view = new QTableView();
    view->setModel(liveTagQuery.model());

    view->show();
    return app.exec();
}
