/****************************************************************************
**
** Copyright (C) 2010-2011 Nokia Corporation and/or its subsidiary(-ies).
** All rights reserved.
** Contact: Nokia Corporation (ivan.frade@nokia.com)
**
** This file is part of QtSparqlTracker library.
**
** $QT_BEGIN_LICENSE:LGPL$
** Commercial Usage
** Licensees holding valid Qt Commercial licenses may use this file in
** accordance with the Qt Commercial License Agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and Nokia.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 2.1 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU Lesser General Public License version 2.1 requirements
** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** In addition, as a special exception, Nokia gives you certain additional
** rights.  These rights are described in the Nokia Qt LGPL Exception
** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3.0 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU General Public License version 3.0 requirements will be
** met: http://www.gnu.org/copyleft/gpl.html.
**
** If you have questions regarding the use of this file, please contact
** Nokia at ivan.frade@nokia.com.
** $QT_END_LICENSE$
**
****************************************************************************/

#include "liveperformancerecorder.h"
#include <QVariant>
#include <QDateTime>

using namespace LivePerformanceProperties;

namespace {
    bool setTimeStampProperty(QObject* obj, const char* name)
    {
        return obj->setProperty(name, QVariant(QDateTime::currentMSecsSinceEpoch()));
    }

    void clearProperty(QObject* obj, const char* name)
    {
        obj->setProperty(name, QVariant());
    }

    bool isPropertySet(const QObject* obj, const char* name)
    {
        return obj->property(name).isValid();
    }
}

LivePerformanceRecorder* LivePerformanceRecorder::create(QObject* parent)
{
    enum State { UNKNOWN, DISABLED, ENABLED };
    static State state = UNKNOWN;
    if (state == UNKNOWN) {
        if (qgetenv("LIVEQUERY_BENCHMARK").isEmpty())
            state = DISABLED;
        else
            state = ENABLED;
    }
    return (state==ENABLED ? new LivePerformanceRecorder(parent) : 0);
}

LivePerformanceRecorder::LivePerformanceRecorder(QObject* parent)
    : QObject(parent)
{
}

void LivePerformanceRecorder::reset()
{
    parent()->setProperty(property_test_updatequerycount, QVariant(0));
    parent()->setProperty(property_test_requerycount, QVariant(0));
}

void LivePerformanceRecorder::markLiveQueryStarted()
{
    setTimeStampProperty(parent(), property_test_livequerystart);
}

void LivePerformanceRecorder::markOntologyQueryFinished()
{
    setTimeStampProperty(parent(), property_test_ontologyquerycomplete);
}

void LivePerformanceRecorder::markOriginalQueryStarted()
{
    setTimeStampProperty(parent(), property_test_originalquerystart);
    clearProperty(parent(), property_test_firstbatchoforiginalqueryavailable);
    parent()->setProperty(property_test_modelbuildtime, QVariant(0));
    parent()->setProperty(property_test_maxmodelblockbuildtime, QVariant(0));
    clearProperty(parent(), property_test_originalqueryprocessingfinished);
}

void LivePerformanceRecorder::markOriginalQueryDataBlockReady()
{
    if (!isPropertySet(parent(), property_test_firstbatchoforiginalqueryavailable)) {
        setTimeStampProperty(parent(), property_test_firstbatchoforiginalqueryavailable);
    }
}

void LivePerformanceRecorder::markOriginalDataBlockToModelWriteStart()
{
    modelUpdateTimer.start();
}

void LivePerformanceRecorder::markOriginalDataBlockToModelWriteEnd()
{
    const int modelUpdateElapsedTime = int(modelUpdateTimer.elapsed());
    if (!isPropertySet(parent(), property_test_firstbatchoforiginaldatainmodel)) {
        setTimeStampProperty(parent(), property_test_firstbatchoforiginaldatainmodel);
    }
    if (modelUpdateElapsedTime > parent()->property(property_test_maxmodelblockbuildtime).toInt())
        parent()->setProperty(property_test_maxmodelblockbuildtime, QVariant(modelUpdateElapsedTime));
    parent()->setProperty(property_test_modelbuildtime,
        QVariant(parent()->property(property_test_modelbuildtime).toInt() + modelUpdateElapsedTime));
    setTimeStampProperty(parent(), property_test_originalqueryprocessingfinished);
}

void LivePerformanceRecorder::markInitialQueryEmitted()
{
    setTimeStampProperty(parent(), property_test_initialdataready);
}

void LivePerformanceRecorder::incrementUpdateQueryCounter()
{
    parent()->setProperty(property_test_updatequerycount,
                          parent()->property(property_test_updatequerycount).toInt() + 1);
}

void LivePerformanceRecorder::incrementReQueryCounter()
{
    parent()->setProperty(property_test_requerycount,
                          parent()->property(property_test_requerycount).toInt() + 1);
}
