/****************************************************************************
**
** Copyright (C) 2010-2011 Nokia Corporation and/or its subsidiary(-ies).
** All rights reserved.
** Contact: Nokia Corporation (ivan.frade@nokia.com)
**
** This file is part of QtSparqlTracker library.
**
** $QT_BEGIN_LICENSE:LGPL$
** Commercial Usage
** Licensees holding valid Qt Commercial licenses may use this file in
** accordance with the Qt Commercial License Agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and Nokia.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 2.1 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU Lesser General Public License version 2.1 requirements
** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** In addition, as a special exception, Nokia gives you certain additional
** rights.  These rights are described in the Nokia Qt LGPL Exception
** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3.0 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU General Public License version 3.0 requirements will be
** met: http://www.gnu.org/copyleft/gpl.html.
**
** If you have questions regarding the use of this file, please contact
** Nokia at ivan.frade@nokia.com.
** $QT_END_LICENSE$
**
****************************************************************************/

#ifndef LIVEPERFORMANCERECORDER__H
#define LIVEPERFORMANCERECORDER__H

#include <QObject>
#include <QElapsedTimer>

// Property names for TrackerLiveQuery class
namespace LivePerformanceProperties {
    // Number of executed update queries
    const char* const property_test_updatequerycount = "test-updatequerycount";
    // Number of executed re-queries
    const char* const property_test_requerycount     = "test-requerycount";
    // Timestamp of the live query start
    const char* const property_test_livequerystart                  = "test-livequerystart";
    // Timestamp of ontology query complete
    const char* const property_test_ontologyquerycomplete           = "test-ontologyquerycomplete";
    // Timestamp of the live query initial data ready
    const char* const property_test_initialdataready                = "test-initialdataready";
    // Timestamp of original query start-up time
    const char* const property_test_originalquerystart              = "test-originalquerystart";
    // Timestamp for first batch of original query data available
    const char* const property_test_firstbatchoforiginalqueryavailable = "test-firstbatchoforiginalqueryavailable";
    // Timestamp for first batch of original query data processed, written in to the model and signaled to client
    const char* const property_test_firstbatchoforiginaldatainmodel = "test-firstbatchoforiginaldatainmodel";
    // Time in MSec spent building the model from the original query
    const char* const property_test_modelbuildtime                  = "test-modelbuildtime";
    // Worst-case time spent building the original model from original query data block
    const char* const property_test_maxmodelblockbuildtime          = "test-maxmodelblockbuildtime";
    // Timestamp of original query result processing finished
    const char* const property_test_originalqueryprocessingfinished = "test-originalqueryprocessingfinished";
}

class LivePerformanceRecorder : public QObject
{
public:
    // Creates and returns an instance of this class if performance counters are enabled,
    // or null if not enabled.
    // The performance counters will be set as QObject properties of object parent and this
    // object will be owned by parent.
    static LivePerformanceRecorder* create(QObject* parent);

    void reset();

    void markLiveQueryStarted();
    void markOntologyQueryFinished();
    void markOriginalQueryStarted();
    void markOriginalQueryDataBlockReady();
    void markOriginalDataBlockToModelWriteStart();
    void markOriginalDataBlockToModelWriteEnd();
    void markInitialQueryEmitted();

    void incrementUpdateQueryCounter();
    void incrementReQueryCounter();

private:
    // Private constructor to enforce use of create
    LivePerformanceRecorder(QObject* parent);

private:
    QElapsedTimer modelUpdateTimer;
};

#endif // LIVEPERFORMANCERECORDER__H
