/****************************************************************************
**
** Copyright (C) 2010-2011 Nokia Corporation and/or its subsidiary(-ies).
** All rights reserved.
** Contact: Nokia Corporation (ivan.frade@nokia.com)
**
** This file is part of QtSparqlTracker library.
**
** $QT_BEGIN_LICENSE:LGPL$
** Commercial Usage
** Licensees holding valid Qt Commercial licenses may use this file in
** accordance with the Qt Commercial License Agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and Nokia.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 2.1 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU Lesser General Public License version 2.1 requirements
** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** In addition, as a special exception, Nokia gives you certain additional
** rights.  These rights are described in the Nokia Qt LGPL Exception
** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3.0 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU General Public License version 3.0 requirements will be
** met: http://www.gnu.org/copyleft/gpl.html.
**
** If you have questions regarding the use of this file, please contact
** Nokia at ivan.frade@nokia.com.
** $QT_END_LICENSE$
**
****************************************************************************/

#ifndef MOVABLEROWMODEL__H
#define MOVABLEROWMODEL__H

#include <QAbstractTableModel>
#include <QtGui/QStandardItemModel>

/**
 * Modification interface for MovableRowModel.
 */
class MovableRowModelModifier
{
public:
    virtual ~MovableRowModelModifier()
    { }
    inline  bool addColumn(int column)
    { return addColumns(column, 1); }
    virtual bool addColumns(int column, int count) = 0;

    virtual void beginAddRowsBatch(int row, int count) = 0;
    virtual void endAddRowsBatch() = 0;
    // addRow and addRowWithData are affected by the batch operations
    inline  bool addRow(int row)
    { return addRows(row, 1); }
    virtual bool addRows(int row, int count) = 0;
    virtual bool addRowWithData(int row, const QAbstractItemModel& sourceModel,
                                int sourceRow) = 0;
    virtual bool addRowWithData(int row, const QVariantList& data) = 0;
    inline  bool deleteRow(int row)
    { return deleteRows(row, 1); }
    virtual bool deleteRows (int row, int count) = 0;
    virtual bool moveRow(int row, int where) = 0;
    virtual bool setValue(const QModelIndex& index, const QVariant& value) = 0;
};

class MovableRowModel : public QAbstractTableModel, public MovableRowModelModifier
{
    Q_OBJECT

public:
    MovableRowModel(QObject* parent = 0);
    ~MovableRowModel();

    // From QAbstractTableModel
    int rowCount(const QModelIndex &parent = QModelIndex()) const;
    int columnCount(const QModelIndex &parent = QModelIndex()) const;
    QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const;
    QModelIndex index(int row, int column, const QModelIndex & parent = QModelIndex()) const;
    QVariant headerData(int section, Qt::Orientation orientation,
                        int role = Qt::DisplayRole) const;
    bool setHeaderData(int section, Qt::Orientation orientation,
                       const QVariant & value, int role = Qt::EditRole);

    // From MovableRowModelModifier
    bool addColumns(int column, int count);

    void beginAddRowsBatch(int row, int count);
    void endAddRowsBatch();

    bool addRows(int row, int count);
    bool addRowWithData(int row, const QAbstractItemModel& sourceModel,
                        int sourceRow);
    bool addRowWithData(int row, const QVariantList& data);
    bool deleteRows(int row, int count);
    bool moveRow(int row, int where);
    bool setValue(const QModelIndex& index, const QVariant& value);

private:
    // Hidden QAbstractItemModel modification interface.
    // Use MovableRowModelModifier interface instead.
    using QAbstractTableModel::insertRow;
    using QAbstractTableModel::insertRows;
    using QAbstractTableModel::insertColumn;
    using QAbstractTableModel::insertColumns;
    using QAbstractTableModel::removeColumn;
    using QAbstractTableModel::removeColumns;
    using QAbstractTableModel::setData;

    QModelIndex toModelIndex(const QModelIndex& index) const;
    QModelIndex toOwnIndex(int row, int column, const QModelIndex& parent) const;
    QModelIndex toOwnIndex(const QModelIndex& index) const;

    void connectDataChanged();
    void disconnectDataChanged();

private slots:
    void convertDataChanged(const QModelIndex& topLeft, const QModelIndex& bottomRight);

private:
    QStandardItemModel model;
    bool batchInProgress; // true if a row adding batch is in progress
};

#endif
