/****************************************************************************
**
** Copyright (C) 2010-2011 Nokia Corporation and/or its subsidiary(-ies).
** All rights reserved.
** Contact: Nokia Corporation (ivan.frade@nokia.com)
**
** This file is part of QtSparqlTracker library.
**
** $QT_BEGIN_LICENSE:LGPL$
** Commercial Usage
** Licensees holding valid Qt Commercial licenses may use this file in
** accordance with the Qt Commercial License Agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and Nokia.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 2.1 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU Lesser General Public License version 2.1 requirements
** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** In addition, as a special exception, Nokia gives you certain additional
** rights.  These rights are described in the Nokia Qt LGPL Exception
** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3.0 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU General Public License version 3.0 requirements will be
** met: http://www.gnu.org/copyleft/gpl.html.
**
** If you have questions regarding the use of this file, please contact
** Nokia at ivan.frade@nokia.com.
** $QT_END_LICENSE$
**
****************************************************************************/

#ifndef ONTOLOGY__H
#define ONTOLOGY__H

#include <QObject>
#include <QHash>
#include <QString>
#include <QSet>

class QSparqlConnection;
class QSparqlResult;

// Class which we can ask ontology information from
// - translates between long and short names of classes
// - translates between predicate names and tracker:ids
class OntologyQuerier : public QObject
{
    Q_OBJECT

public:
    OntologyQuerier(QSparqlConnection& conn, QObject* parent = 0);
    ~OntologyQuerier();

    void addPredicate(const QString& shortPredicate);
    void addClass(const QString& shortClass);

    void start();

    QString longClassToShort(const QString& klass) const;
    const QHash<QString, int>& shortPredicateToIdMap() const;

    QList<QString> longClasses() const;

private Q_SLOTS:
    void ontologyQueryFinished();

Q_SIGNALS:
    void finished();

private:
    QSparqlConnection& conn;
    bool classesAdded;
    QString ontologySnippet;
    QSparqlResult* ontologyResult;

    // The results of the query
    QHash<QString, int> shortToId; // for predicates
    QHash<QString, QString> longToShort; // for classes
};

#endif
