/****************************************************************************
**
** Copyright (C) 2010-2011 Nokia Corporation and/or its subsidiary(-ies).
** All rights reserved.
** Contact: Nokia Corporation (ivan.frade@nokia.com)
**
** This file is part of QtSparqlTracker library.
**
** $QT_BEGIN_LICENSE:LGPL$
** Commercial Usage
** Licensees holding valid Qt Commercial licenses may use this file in
** accordance with the Qt Commercial License Agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and Nokia.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 2.1 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU Lesser General Public License version 2.1 requirements
** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** In addition, as a special exception, Nokia gives you certain additional
** rights.  These rights are described in the Nokia Qt LGPL Exception
** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3.0 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU General Public License version 3.0 requirements will be
** met: http://www.gnu.org/copyleft/gpl.html.
**
** If you have questions regarding the use of this file, please contact
** Nokia at ivan.frade@nokia.com.
** $QT_END_LICENSE$
**
****************************************************************************/

#ifndef TRACKERLIVEQUERY__H
#define TRACKERLIVEQUERY__H

#ifndef Q_EXPORT_QTSPARQLTRACKER
# if defined(QTSPARQLTRACKER_BUILD)
#  define Q_EXPORT_QTSPARQLTRACKER Q_DECL_EXPORT
# else
#  define Q_EXPORT_QTSPARQLTRACKER Q_DECL_IMPORT
# endif
#endif

#include <QObject>
#include <QVariant>

class QSparqlConnection;
class QSparqlQuery;

class QString;
class QAbstractItemModel;

class TrackerPartialUpdater;
class TrackerFullUpdater;
class TrackerLiveQueryPrivate;

class Q_EXPORT_QTSPARQLTRACKER TrackerLiveQuery : public QObject
{
    Q_OBJECT
    Q_PROPERTY(QAbstractItemModel* model READ model)

public:
    struct CollationColumn
    {
        int column;
        QVariant::Type type;
        Qt::SortOrder order;

        CollationColumn(int column, QVariant::Type type,
                        Qt::SortOrder order = Qt::AscendingOrder)
        : column(column), type(type), order(order)
        {
        }
    };

    /// Constructs a TrackerLiveQuery.  First the model is filled with the
    /// results of \a originalQuery, and the data is kept up to date with
    /// updaters (TrackerPartialUpdater and TrackerFullUpdater) which are added
    /// to this TrackerLiveQuery with addUpdater().  \a columnCount is the
    /// number of columns in the original query, and \a identityColumns are the
    /// identity columns of the model. \a connection is the QSparqlConnection
    /// for executing the queries (the original query and the queries generated
    /// by the updaters).
    TrackerLiveQuery(const QString& originalQuery,
                     int columnCount,
                     const QList<int>& identityColumns,
                     QSparqlConnection& connection);

    /// Constructs a TrackerLiveQuery.  First the model is filled with the
    /// results of \a originalQuery, and the data is kept up to date with
    /// updaters (TrackerPartialUpdater and TrackerFullUpdater) which are added
    /// to this TrackerLiveQuery with addUpdater().  \a columnCount is the
    /// number of columns in the original query, and \a connection is the
    /// QSparqlConnection for executing the queries (the original query and the
    /// queries generated by the updaters).
    TrackerLiveQuery(const QSparqlQuery& originalQuery,
                     int columnCount,
                     const QList<int>& identityColumns,
                     QSparqlConnection& connection);

    /// Constructs a TrackerLiveQuery.  First the model is filled with the
    /// results of \a originalQuery, and the data is kept up to date with
    /// updaters (TrackerPartialUpdater and TrackerFullUpdater) which are added
    /// to this TrackerLiveQuery with addUpdater().  \a columnCount is the
    /// number of columns in the original query, and \a connection is the
    /// QSparqlConnection for executing the queries (the original query and the
    /// queries generated by the updaters).  \deprecated Use the constructor
    /// which takes the identity columns as a parameter.
    TrackerLiveQuery(const QString& originalQuery,
                     int columnCount,
                     QSparqlConnection& connection);

    /// Constructs a TrackerLiveQuery.  First the model is filled with the
    /// results of \a originalQuery, and the data is kept up to date with
    /// updaters (TrackerPartialUpdater and TrackerFullUpdater) which are added
    /// to this TrackerLiveQuery with addUpdater().  \a columnCount is the
    /// number of columns in the original query, and \a connection is the
    /// QSparqlConnection for executing the queries (the original query and the
    /// queries generated by the updaters).  \deprecated Use the constructor
    /// which takes the identity columns as a parameter.
    TrackerLiveQuery(const QSparqlQuery& originalQuery,
                     int columnCount,
                     QSparqlConnection& connection);

    /// Destroys the TrackerLiveQuery.
    ~TrackerLiveQuery();

    /// Sets \a columns to be used as identity columns when merging new data
    /// into the model.  The values in the identity columns determine if two
    /// rows are the same. The model cannot have two rows with the same
    /// identity. A limitation: the contents of the identity columns must be
    /// integers.  \deprecated Use TrackerLiveQuery constructor which takes the
    /// identity columns as a parameter.
    void setIdentityColumns(const QList<int>& columns);

    /// Sets \a columns to define the sorting of the rows in the model.  The
    /// sorting is used when merging new data into the model.
    void setCollationColumns(const QList<CollationColumn>& columns);

    /// Adds a TrackerPartialUpdater to keep the data model up to date.  It will
    /// re-run a limited query to update the data in the model.
    void addUpdater(const TrackerPartialUpdater& updater);

    /// Adds a TrackerFullUpdater to keep the data model up to date.  It will
    /// re-run the original query to update the data in the model.
    void addUpdater(const TrackerFullUpdater& updater);

    /// Returns a model that can be used e.g., in UIs. The model will change its
    /// data and emit the corresponding QAbstractItemModel signals when data in
    /// the database changes.
    QAbstractItemModel* model() const;

public Q_SLOTS:
    /// Must be called after giving data (setIdentityColumns(),
    /// setCollationColumns(), addUpdater()) is done and the model can start its
    /// operation.  After calling start(), the user must spin the event loop
    /// regularly for the TrackerLiveQuery to operate correctly.
    bool start();

    /// Pauses or resumes the live updates.  When the updates are disabled, the
    /// TrackerLiveQuery gathers data about update queries to be run, but
    /// doesn't run them.  The queries are eventually ran when updates are
    /// enabled the next time.  By default, the updates are enabled.
    void setUpdatesEnabled(bool enable);

Q_SIGNALS:
    /// Emitted when the model is filled with the initial query data (the result
    /// of original query passed to the TrackerLiveQuery()).
    void initialQueryFinished();

private:
    TrackerLiveQueryPrivate* d;
};

struct TrackerPartialUpdaterPrivate;

class Q_EXPORT_QTSPARQLTRACKER TrackerPartialUpdater
{
public:
    enum PlaceInSignal {
        Subject = 0,
        Object
    };
    /// Constructs a TrackerLiveQueryUpdater which will run an update query
    /// specified by the \a updateQuery.  \a updateQuery must contain exactly
    /// one "%FILTER" placeholder where the FILTER statement is inserted.  The
    /// user specifies to which changes the TrackerPartialUpdater reacts to by
    /// calling watchClass() one or multiple times.
    TrackerPartialUpdater(const QString& updateQuery);
    ~TrackerPartialUpdater();

    TrackerPartialUpdater(const TrackerPartialUpdater& other);
    TrackerPartialUpdater& operator=(const TrackerPartialUpdater& other);

    /// Defines when to run the update query (when the class \a klass and one of
    /// its \a predicates changes), what kind of update query to run (the FILTER
    /// statement is constructed from \a filterSnippet strings which contain a
    /// "%LIST" placeholder for an id list, \a placeInSignal indicating whether
    /// to put subjects or objects to the list), and which rows are affected by
    /// the update query (\a idColumn contains one of the tracker:ids in the
    /// signal).  An empty predicate list means "all predicates".
    void watchClass(const QString& klass, const QStringList& predicates,
                    const QString& filterSnippet, PlaceInSignal placeInSignal,
                    int idColumn);

    /// The same as watchClass(), but only watches insertion of \a predicates to
    /// an object belonging to class \a klass.
    void watchClassForInserts(const QString& klass,
                              const QStringList& predicates,
                              const QString& filterSnippet,
                              PlaceInSignal placeInSignal,
                              int idColumn);

    /// The same as watchClass(), but only watches insertion of \a predicates to
    /// an object belonging to class \a klass.
    void watchClassForDeletes(const QString& klass,
                              const QStringList& predicates,
                              const QString& filterSnippet,
                              PlaceInSignal placeInSignal,
                              int idColumn);

private:
    TrackerPartialUpdaterPrivate* d;
    friend class TrackerLiveQuery;
};

struct TrackerFullUpdaterPrivate;

class Q_EXPORT_QTSPARQLTRACKER TrackerFullUpdater
{
public:
    /// Constructs the TrackerFullUpdater.  The user specifies to which changes
    /// the TrackerFullUpdater reacts to by calling watchClass() one or multiple
    /// times.
    TrackerFullUpdater();
    ~TrackerFullUpdater();
    TrackerFullUpdater(const TrackerFullUpdater& other);
    TrackerFullUpdater& operator=(const TrackerFullUpdater& other);

    /// Defines when to re-run the original query (when the class \a klass and
    /// one of its \a predicates changes).  Empty predicate list means "all
    /// predicates".  The original query is given to the TrackerLiveQuery
    /// constructor.
    void watchClass(const QString& klass, const QStringList& predicates);

    /// Defines when to re-run the original query (when one of the \a predicates
    /// is inserted for an object of the class \a klass).  Empty predicate list
    /// means "all predicates".  The original query is given to the
    /// TrackerLiveQuery constructor.
    void watchClassForInserts(const QString& klass,
                              const QStringList& predicates);

    /// Defines when to re-run the original query (when one of the \a predicates
    /// is deleted for an object of the class \a klass).  Empty predicate list
    /// means "all predicates".  The original query is given to the
    /// TrackerLiveQuery constructor.
    void watchClassForDeletes(const QString& klass,
                              const QStringList& predicates);

private:
    TrackerFullUpdaterPrivate* d;
    friend class TrackerLiveQuery;
};

#endif
