/****************************************************************************
**
** Copyright (C) 2010-2011 Nokia Corporation and/or its subsidiary(-ies).
** All rights reserved.
** Contact: Nokia Corporation (ivan.frade@nokia.com)
**
** This file is part of QtSparqlTracker library.
**
** $QT_BEGIN_LICENSE:LGPL$
** Commercial Usage
** Licensees holding valid Qt Commercial licenses may use this file in
** accordance with the Qt Commercial License Agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and Nokia.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 2.1 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU Lesser General Public License version 2.1 requirements
** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** In addition, as a special exception, Nokia gives you certain additional
** rights.  These rights are described in the Nokia Qt LGPL Exception
** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3.0 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU General Public License version 3.0 requirements will be
** met: http://www.gnu.org/copyleft/gpl.html.
**
** If you have questions regarding the use of this file, please contact
** Nokia at ivan.frade@nokia.com.
** $QT_END_LICENSE$
**
****************************************************************************/

#ifndef UPDATER__H
#define UPDATER__H

#include "trackerlivequery.h"

#include <TrackerChangeNotifier>

#include <QSet>
#include <QString>

class UpdateRule;

// Describes a change the updater reacts to.
struct Change
{
    enum Type
    {
        Insert = 1 << 0,
        Delete = 1 << 1
    };

    Change(const QString& klass, const QSet<QString>& predicates,
           unsigned int type)
    : klass(klass), predicates(predicates), type(type)
    {
    }
    // Compiler-generated copy constructor, assignment operator and destructor
    // are OK for this class

    bool hasPredicateId(int predicateId) const
    {
        // No predicate ids means all predicate ids
        return( predicateIds.isEmpty() || predicateIds.contains(predicateId) );
    }

    QString klass;
    QSet<QString> predicates;
    QSet<int> predicateIds;
    unsigned int type;
};

struct PartialUpdateSchema
{
    PartialUpdateSchema(const Change& change,
                        const QString& filterSnippet,
                        TrackerPartialUpdater::PlaceInSignal idPlace,
                        int idColumn);
    // Compiler-generated copy constructor, assignment operator and destructor
    // are OK for this class

    bool storeIds(const QString& classShort,
            const QList<TrackerChangeNotifier::Quad>& deletes,
            const QList<TrackerChangeNotifier::Quad>& inserts);
    void storeSubjectOrObjectIds
            (bool& idsAdded, const QList<TrackerChangeNotifier::Quad>& quads);

    // Which change to react to
    Change change;

    // Form the update query by filtering...
    // FILTER (filterSnippet.replace("%LIST", (list gathered from idPlace)))
    QString filterSnippet;
    TrackerPartialUpdater::PlaceInSignal idPlace;

    // How to identify affected rows: this column contains one of the id's that
    // was put to the filtering list.
    int idColumn;

    // Dynamic data: the list of ID's used in the filtering when the update
    // query will be run next time.
    QSet<int> ids;
};

struct TrackerPartialUpdaterPrivate
{
    TrackerPartialUpdaterPrivate(const QString& updateQuery);
    TrackerPartialUpdaterPrivate(const TrackerPartialUpdaterPrivate& other);
    ~TrackerPartialUpdaterPrivate();

    void watchClass(const QString& classShort, const QStringList& predicates,
                    unsigned int changeType,
                    const QString& filterSnippet,
                    TrackerPartialUpdater::PlaceInSignal placeInSignal,
                    int idColumn);
    void convertPredicates(const QHash<QString, int>& predicateToId);
    bool storeIds(const QString& classShort,
                  const QList<TrackerChangeNotifier::Quad>& deletes,
                  const QList<TrackerChangeNotifier::Quad>& inserts);

    QString buildQuery(QList<UpdateRule>& rules);

    // One query
    QString updateQuery;

    // Multiple (class, predicates, filterSnippet, idType, idColumn) data elements
    QList<PartialUpdateSchema*> schemas; // owned
};

struct TrackerFullUpdaterPrivate
{
    TrackerFullUpdaterPrivate()
    {
    }
    // Compiler-generated copy constructor, assignment operator and destructor
    // are OK for this class

    void watchClass(const QString& classShort, const QStringList& predicates,
                    unsigned int changeType);
    void convertPredicates(const QHash<QString, int>& predicateToId);
    bool checkSignal(const QString& classShort,
                     const QList<TrackerChangeNotifier::Quad>& deletes,
                     const QList<TrackerChangeNotifier::Quad>& inserts);

    // Which changes to react to
    QList<Change> changes;
};

#endif
