/****************************************************************************
**
** Copyright (C) 2010-2011 Nokia Corporation and/or its subsidiary(-ies).
** All rights reserved.
** Contact: Nokia Corporation (ivan.frade@nokia.com)
**
** This file is part of QtSparqlTracker library.
**
** $QT_BEGIN_LICENSE:LGPL$
** Commercial Usage
** Licensees holding valid Qt Commercial licenses may use this file in
** accordance with the Qt Commercial License Agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and Nokia.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 2.1 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU Lesser General Public License version 2.1 requirements
** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** In addition, as a special exception, Nokia gives you certain additional
** rights.  These rights are described in the Nokia Qt LGPL Exception
** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3.0 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU General Public License version 3.0 requirements will be
** met: http://www.gnu.org/copyleft/gpl.html.
**
** If you have questions regarding the use of this file, please contact
** Nokia at ivan.frade@nokia.com.
** $QT_END_LICENSE$
**
****************************************************************************/

#ifndef USERMODIFIABLEMODEL__H
#define USERMODIFIABLEMODEL__H

// Merges changes from these 2 sources together:
// 1) User's modifications to the model
// 2) Updates performed by the TrackerLiveQuery

// Also, when updating the model because of the merge algorithm, groups many
// rowsInserted and rowsRemoved signals together.

// Is it ugly that these 2 responsibilities are in the same class?
// 1) Yes
// 2) No, because otherwise we would need to group and split signals.  Somebody
// needs to group multiple rowsInserted() into one big rowsInserted().  But what
// if the user-modifiability layer knows that there is a user-inserted row in
// the middle of that range?  If the signal grouping is already done, the
// user-modifiability layer should split the signal into two.  And if the signal
// grouping is done below, the user-modifiability layer needs to know about it
// (it needs to have the same beginInserions/endInsertions type of an api).

#include "movablerowmodel.h"
#include "identity.h"

#include <QPersistentModelIndex>
#include <QList>
#include <QHash>
#include <QSet>

class UpdateRule;

// TODO: private inheritance
class UserModifiableModel : public MovableRowModel
{
    Q_OBJECT
public:
    UserModifiableModel();
    ~UserModifiableModel();

    void setIdentityColumns(const QList<int>& identityColumns);

    // Interface used to push original query results into the model
    bool addRowWithData(int row, const QVariantList& data);

    // Interface used by the merge algorithm
    bool hasRowWithIdentity(const IdentityList& identity) const;
    QPersistentModelIndex identityToRow(const IdentityList& identity) const;

    void beginLazyRowRemovals();
    bool lazyRemoveRow(const IdentityList& identity);
    void endLazyRowRemovals();

    void beginLazyRowInsertions();
    void lazyInsertRow(int row, const QVariantList& data);
    void endLazyRowInsertions();

    QLinkedList<IdentityList> affectedRows(const QList<UpdateRule>& rules) const;

public:
    // From QAbstractItemModel
    bool removeRows(int row, int count, const QModelIndex & parent = QModelIndex());

private:
    // hide MovableRowModel functions
    using MovableRowModel::addRows;
    using MovableRowModel::deleteRows;

    // helper functions
    void startNewInsertBatch(int row, const IdentityList& identity,
                             const QVariantList& data);
    int insertCurrentBatchIfAny();

    // additional data structures

    QList<int> identityColumns;

    // Index over model: identity -> row (persisten model index)
    QHash<IdentityList, QPersistentModelIndex> identityToRowCache;

    int insertBatchLocation;
    QList<QVariantList> insertBatchData;
    QList<IdentityList> insertBatchIdentities;

    QSet<IdentityList> userRemovedRows;
};

#endif
