/****************************************************************************
**
** Copyright (C) 2010-2011 Nokia Corporation and/or its subsidiary(-ies).
** All rights reserved.
** Contact: Nokia Corporation (ivan.frade@nokia.com)
**
** This file is part of QtSparqlTracker library.
**
** $QT_BEGIN_LICENSE:LGPL$
** GNU Lesser General Public License Usage
** This file may be used under the terms of the GNU Lesser General Public
** License version 2.1 as published by the Free Software Foundation and
** appearing in the file LICENSE.LGPL included in the packaging of this
** file. Please review the following information to ensure the GNU Lesser
** General Public License version 2.1 requirements will be met:
** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** In addition, as a special exception, Nokia gives you certain additional
** rights. These rights are described in the Nokia Qt LGPL Exception
** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU General
** Public License version 3.0 as published by the Free Software Foundation
** and appearing in the file LICENSE.GPL included in the packaging of this
** file. Please review the following information to ensure the GNU General
** Public License version 3.0 requirements will be met:
** http://www.gnu.org/copyleft/gpl.html.
**
** Other Usage
** Alternatively, this file may be used in accordance with the terms and
** conditions contained in a signed written agreement between you and Nokia.
**
** If you have questions regarding the use of this file, please contact
** Nokia at ivan.frade@nokia.com.
** $QT_END_LICENSE$
**
****************************************************************************/

#include "../utils/livetesthelpers.h"
#include "../utils/updatewaiter.h"

#include "TrackerLiveQuery"

#include <QtTest/QtTest>
#include <QtSparql/QtSparql>
#include <QList>
#include <QString>

//const QString qtest(qTableName( "qtest", __FILE__ )); // FIXME: what's this

//TESTED_FILES=

/*

This test contains a live model with aggregates:

column 0: tracker:id of an artist
column 1: artist name of the artist
column 2: no. of songs for the artist
column 3: min song length for the artist
column 4: max song length for the artist

The relationship goes this way: song --(performer)--> artist.

*/

class tst_TrackerLiveAggregates : public QObject
{
    Q_OBJECT

public:
    tst_TrackerLiveAggregates();
    virtual ~tst_TrackerLiveAggregates();

public slots:
    void initTestCase();
    void cleanupTestCase();
    void init();
    void cleanup();

private slots:
    void initialModel();

    void primaryResourceAdded();
    void primaryResourceRemoved();

    void aggregatedResourceAdded();
    void aggregatedResourceRemovedExplicit();
    void aggregatedResourceRemoved();
    void aggregatedResourceRelationshipChanged();
    void aggregatedResourceChanged();
/*
    void primaryAndAggregatedResourceAdded();
*/
private:
    // Object under test
    TrackerLiveQuery* liveQuery;
    QSparqlConnection* conn;
    QAbstractItemModel* model;
};

Q_DECLARE_METATYPE(QModelIndex)

tst_TrackerLiveAggregates::tst_TrackerLiveAggregates()
: liveQuery(0), conn(0), model(0)
{
}

tst_TrackerLiveAggregates::~tst_TrackerLiveAggregates()
{
}

void tst_TrackerLiveAggregates::initTestCase()
{
    qRegisterMetaType<QModelIndex>("QModelIndex");
    // Clean up leftover data from previous test run
    cleanup();
}

void tst_TrackerLiveAggregates::cleanupTestCase()
{
}

void tst_TrackerLiveAggregates::init()
{
    UpdateWaiter waiterMusicPiece(QString("http://www.tracker-project.org/temp/nmm#MusicPiece"), 27);
    UpdateWaiter waiterArtist(QString("http://www.tracker-project.org/temp/nmm#Artist"), 12);
    // Insert initial test data
    executeQuery(QSparqlQuery(
                     "insert { "
                     "<tracker-live-tests> a nie:InformationElement . "
                     "<artist001> a nmm:Artist ; " // 0 songs
                     "  nie:isLogicalPartOf <tracker-live-tests> ; "
                     "  nmm:artistName \"aaa\" . "
                     "<artist002> a nmm:Artist ; " // 1 song
                     "  nie:isLogicalPartOf <tracker-live-tests> ; "
                     "  nmm:artistName \"bbb\" . "
                     "<artist003> a nmm:Artist ; " // 2 songs
                     "  nie:isLogicalPartOf <tracker-live-tests> ; "
                     "  nmm:artistName \"ccc\" . "
                     "<artist004> a nmm:Artist ; " // 3 songs
                     "  nie:isLogicalPartOf <tracker-live-tests> ; "
                     "  nmm:artistName \"ddd\" . "
                     "<song002a> a nmm:MusicPiece ; "
                     "  nie:isLogicalPartOf <tracker-live-tests> ; "
                     "  nmm:performer <artist002> ; "
                     "  nfo:duration 100 . "
                     "<song003a> a nmm:MusicPiece ; "
                     "  nie:isLogicalPartOf <tracker-live-tests> ; "
                     "  nmm:performer <artist003> ; "
                     "  nfo:duration 50 . "
                     "<song003b> a nmm:MusicPiece ; "
                     "  nie:isLogicalPartOf <tracker-live-tests> ; "
                     "  nmm:performer <artist003> ; "
                     "  nfo:duration 20 . "
                     "<song004a> a nmm:MusicPiece ; "
                     "  nie:isLogicalPartOf <tracker-live-tests> ; "
                     "  nmm:performer <artist004> ; "
                     "  nfo:duration 500 . "
                     "<song004b> a nmm:MusicPiece ; "
                     "  nie:isLogicalPartOf <tracker-live-tests> ; "
                     "  nmm:performer <artist004> ; "
                     "  nfo:duration 500 . "
                     "<song004c> a nmm:MusicPiece ; "
                     "  nie:isLogicalPartOf <tracker-live-tests> ; "
                     "  nmm:performer <artist004> ; "
                     "  nfo:duration 400 . "
                     "<song000a> a nmm:MusicPiece ; "
                     "  nie:isLogicalPartOf <tracker-live-tests> ; "
                     "  nfo:duration 444 . "
                     "}",
                     QSparqlQuery::InsertStatement),
                 false);
    // Wait until all inserted data is in tracker
    QVERIFY(waiterMusicPiece.waitForAllUpdated());
    QVERIFY(waiterArtist.waitForAllUpdated());

    // Create the object under test
    QString query("select tracker:id(?a) ?an count(?mp) min(?l) max(?l)"
                  "{ "
                  "?a a nmm:Artist ; "
                  "  nie:isLogicalPartOf <tracker-live-tests> ; "
                  "  nmm:artistName ?an ."
                  "OPTIONAL { "
                  "  ?mp a nmm:MusicPiece ; "
                  "    nie:isLogicalPartOf <tracker-live-tests> ; "
                  "    nmm:performer ?a ; "
                  "    nfo:duration ?l . "
                  "} "
                  "%1"
                  "} GROUP BY ?a ORDER BY ?an");
    conn = new QSparqlConnection("QTRACKER_DIRECT");
    // 0 is the only identity column; this model is essentially about artists
    // and their properties.
    liveQuery = new TrackerLiveQuery(query.arg(""), 5, QList<int>() << 0, *conn);

    TrackerPartialUpdater up(query.arg("%FILTER"));
    up.watchClass("nmm:Artist", QStringList(),
                  "tracker:id(?a) in %LIST", TrackerPartialUpdater::Subject, 0);

    up.watchClass("nmm:MusicPiece",
                  QStringList() << "nmm:performer",
                  "tracker:id(?a) in %LIST", TrackerPartialUpdater::Object, 0);

    up.watchClass("nmm:MusicPiece",
                  QStringList() << "nfo:duration",
                  "EXISTS { "
                  "  ?mp_filter a nmm:MusicPiece ; "
                  "    nie:isLogicalPartOf <tracker-live-tests> ; "
                  "    nmm:performer ?a . "
                  "  FILTER(?mp_filter in %LIST) "
                  " } ", TrackerPartialUpdater::Subject, -1);
    // Note: here we don't require that mp_filter has a length; this
    // accommodates for the cases where the length of the music piece is
    // removed. The main query takes into account only music pieces which have a
    // length, so the data for the artist will be correct. (Only the filtering,
    // for which artists we re-query the data, changes.)
    liveQuery->addUpdater(up);

    TrackerFullUpdater fullUp;
    // If a music piece is removed, we don't have a choice but we need to re-run
    // the whole query.
    fullUp.watchClassForDeletes("nmm:MusicPiece",
                                QStringList() << "rdf:type");
    liveQuery->addUpdater(fullUp);

    liveQuery->setCollationColumns(QList<TrackerLiveQuery::CollationColumn>() <<
                                   TrackerLiveQuery::CollationColumn(1, QVariant::String));
    liveQuery->start();

    // store this for convenience
    model = liveQuery->model();
}

void tst_TrackerLiveAggregates::cleanup()
{
    // Delete the tested object
    delete liveQuery;
    liveQuery = 0;
    model = 0;
    delete conn;
    conn = 0;

    // Clean up test data
    executeQuery(QSparqlQuery(
                     "delete { "
                     "?u a rdfs:Resource . } "
                     "where { "
                     "?u nie:isLogicalPartOf <tracker-live-tests> . "
                     "} "
                     "delete {"
                     "<tracker-live-tests> a nie:InformationElement . "
                     "}",
                     QSparqlQuery::DeleteStatement));
}

void tst_TrackerLiveAggregates::initialModel()
{
    // Ignore the return value; we'll do the checks here
    waitUntilModelFilled(liveQuery, 4);

    QCOMPARE(model->rowCount(), 4);
    QCOMPARE(model->columnCount(), 5);

    QStringList data = getRows(model, QList<int>() << 1 << 2 << 3 << 4);
    QCOMPARE(data.at(0), QString("aaa:0:::"));
    QCOMPARE(data.at(1), QString("bbb:1:100:100:"));
    QCOMPARE(data.at(2), QString("ccc:2:20:50:"));
    QCOMPARE(data.at(3), QString("ddd:3:400:500:"));
}

void tst_TrackerLiveAggregates::primaryResourceAdded()
{
    QVERIFY(waitUntilModelFilled(liveQuery, 4));

    QStringList oldData = getRows(model, QList<int>() << 1 << 2 << 3 << 4);

    // Insert a resource
    QSignalSpy spy(model, SIGNAL(rowsInserted(const QModelIndex&, int, int)));
    executeQuery(QSparqlQuery(QString(
                                  "insert {"
                                  "<addeduri001> a nmm:Artist ; "
                                  "nie:isLogicalPartOf <tracker-live-tests> ; "
                                  "nmm:artistName \"baa\" . "
                                  "}"),
                              QSparqlQuery::InsertStatement));
    QTime timer;
    timer.start();
    while(timer.elapsed() < 5000 && spy.count() < 1)
        QTest::qWait(100);

    // Verify the inserted row
    QCOMPARE(spy.count(), 1);
    QList<QVariant> parameters = spy.takeFirst();
    QModelIndex ix = parameters.at(0).value<QModelIndex>();
    int start = parameters.at(1).value<int>();
    int end = parameters.at(2).value<int>();

    int row = 1;
    QCOMPARE(ix, QModelIndex());
    QCOMPARE(start, row);
    QCOMPARE(end, row);

    // Verify the data of the rest of the model
    QStringList newData = getRows(model, QList<int>() << 1 << 2 << 3 << 4);
    QCOMPARE(newData.at(row), QString("baa:0:::"));
    QCOMPARE(newData.size(), oldData.size() + 1);
    for (int i = 0; i < row; ++i) {
        QCOMPARE(newData.at(i), oldData.at(i));
    }
    for (int i = row + 1; i < newData.size(); ++i) {
        QCOMPARE(newData.at(i), oldData.at(i-1));
    }
}

void tst_TrackerLiveAggregates::primaryResourceRemoved()
{
    QVERIFY(waitUntilModelFilled(liveQuery, 4));

    QStringList oldData = getRows(model, QList<int>() << 1 << 2 << 3 << 4);

    // Insert a resource
    QSignalSpy spy(model, SIGNAL(rowsRemoved(const QModelIndex&, int, int)));
    executeQuery(QSparqlQuery(QString(
                                  "delete {"
                                  "<artist003> a rdfs:Resource . "
                                  "}"),
                              QSparqlQuery::DeleteStatement));
    QTime timer;
    timer.start();
    while(timer.elapsed() < 5000 && spy.count() < 1)
        QTest::qWait(100);

    // Verify the deleted row
    QCOMPARE(spy.count(), 1);
    QList<QVariant> parameters = spy.takeFirst();
    QModelIndex ix = parameters.at(0).value<QModelIndex>();
    int start = parameters.at(1).value<int>();
    int end = parameters.at(2).value<int>();

    int row = 2;
    QCOMPARE(ix, QModelIndex());
    QCOMPARE(start, row);
    QCOMPARE(end, row);

    // Verify the data of the rest of the model
    QStringList newData = getRows(model, QList<int>() << 1 << 2 << 3 << 4);
    QCOMPARE(newData.size(), oldData.size() - 1);
    for (int i = 0; i < row; ++i) {
        QCOMPARE(newData.at(i), oldData.at(i));
    }
    for (int i = row; i < newData.size(); ++i) {
        QCOMPARE(newData.at(i), oldData.at(i+1));
    }
}

void tst_TrackerLiveAggregates::aggregatedResourceAdded()
{
    QVERIFY(waitUntilModelFilled(liveQuery, 4));

    QStringList oldData = getRows(model, QList<int>() << 1 << 2 << 3 << 4);

    // Insert a resource
    QSignalSpy spy(model,
                   SIGNAL(dataChanged(const QModelIndex&, const QModelIndex&)));
    executeQuery(QSparqlQuery(QString(
                                  "insert {"
                                  "<addedmp001> a nmm:MusicPiece ; "
                                  "  nie:isLogicalPartOf <tracker-live-tests> ; "
                                  "  nfo:duration 60 ; "
                                  "  nmm:performer <artist003> ."
                                  "}"),
                              QSparqlQuery::InsertStatement));
    QTime timer;
    timer.start();
    while(timer.elapsed() < 5000 && spy.count() < 2)
        QTest::qWait(100);

    // Verify the modified row
    QCOMPARE(spy.count(), 2);

    QSet<int> columns;
    int row = 2;
    while (spy.count()) {
        QList<QVariant> parameters = spy.takeFirst();
        QModelIndex topLeft = parameters.at(0).value<QModelIndex>();
        QModelIndex bottomRight = parameters.at(1).value<QModelIndex>();
        QCOMPARE(topLeft.row(), row);
        QCOMPARE(bottomRight.row(), row);
        QCOMPARE(topLeft.column(), bottomRight.column());
        columns.insert(topLeft.column());
    }
    QCOMPARE(columns.size(), 2);
    QVERIFY(columns.contains(2));
    QVERIFY(columns.contains(4));

    // Verify the data of the rest of the model
    QStringList newData = getRows(model, QList<int>() << 1 << 2 << 3 << 4);
    QCOMPARE(newData.size(), oldData.size());
    QCOMPARE(newData.at(row), QString("ccc:3:20:60:"));
    for (int i = 0; i < row; ++i) {
        QCOMPARE(newData.at(i), oldData.at(i));
    }
    for (int i = row + 1; i < newData.size(); ++i) {
        QCOMPARE(newData.at(i), oldData.at(i));
    }
}

void tst_TrackerLiveAggregates::aggregatedResourceRemovedExplicit()
{
    QVERIFY(waitUntilModelFilled(liveQuery, 4));

    QStringList oldData = getRows(model, QList<int>() << 1 << 2 << 3 << 4);
    QSignalSpy spy(model,
                   SIGNAL(dataChanged(const QModelIndex&, const QModelIndex&)));

    // Delete a resource

    // Note: here we delete the triple "<song003a> nmm:performer <artist003>"
    // explicitly. If we don't, this won't work.
    executeQuery(QSparqlQuery(QString(
                                  "delete {"
                                  "<song003a> nmm:performer ?a ; "
                                  "  a rdfs:Resource . "
                                  "} where { "
                                  "<song003a> nmm:performer ?a . "
                                  "}"),
                              QSparqlQuery::DeleteStatement));
    QTime timer;
    timer.start();
    while(timer.elapsed() < 5000 && spy.count() < 2)
        QTest::qWait(100);

    // Verify the modified row
    QCOMPARE(spy.count(), 2);

    QSet<int> columns;
    int row = 2;
    while (spy.count()) {
        QList<QVariant> parameters = spy.takeFirst();
        QModelIndex topLeft = parameters.at(0).value<QModelIndex>();
        QModelIndex bottomRight = parameters.at(1).value<QModelIndex>();
        QCOMPARE(topLeft.row(), row);
        QCOMPARE(bottomRight.row(), row);
        QCOMPARE(topLeft.column(), bottomRight.column());
        columns.insert(topLeft.column());
    }
    QCOMPARE(columns.size(), 2);
    QVERIFY(columns.contains(2));
    QVERIFY(columns.contains(4));

    // Verify the data of the rest of the model
    QStringList newData = getRows(model, QList<int>() << 1 << 2 << 3 << 4);
    QCOMPARE(newData.size(), oldData.size());
    QCOMPARE(newData.at(row), QString("ccc:1:20:20:"));
    for (int i = 0; i < row; ++i) {
        QCOMPARE(newData.at(i), oldData.at(i));
    }
    for (int i = row + 1; i < newData.size(); ++i) {
        QCOMPARE(newData.at(i), oldData.at(i));
    }
}

void tst_TrackerLiveAggregates::aggregatedResourceRemoved()
{
    QVERIFY(waitUntilModelFilled(liveQuery, 4));

    QStringList oldData = getRows(model, QList<int>() << 1 << 2 << 3 << 4);
    QSignalSpy spy(model,
                   SIGNAL(dataChanged(const QModelIndex&, const QModelIndex&)));

    // Delete a resource

    // Note: here we *don't* delete the triple "<song003a> nmm:performer
    // <artist003>" explicitly.
    executeQuery(QSparqlQuery(QString(
                                  "delete {"
                                  "<song003a> a rdfs:Resource . "
                                  "}"),
                              QSparqlQuery::DeleteStatement));
    QTime timer;
    timer.start();
    while(timer.elapsed() < 5000 && spy.count() < 2)
        QTest::qWait(100);

    QCOMPARE(spy.count(), 2);

    QSet<int> columns;
    int row = 2;
    while (spy.count()) {
        QList<QVariant> parameters = spy.takeFirst();
        QModelIndex topLeft = parameters.at(0).value<QModelIndex>();
        QModelIndex bottomRight = parameters.at(1).value<QModelIndex>();
        QCOMPARE(topLeft.row(), row);
        QCOMPARE(bottomRight.row(), row);
        QCOMPARE(topLeft.column(), bottomRight.column());
        columns.insert(topLeft.column());
    }
    QCOMPARE(columns.size(), 2);
    QVERIFY(columns.contains(2));
    QVERIFY(columns.contains(4));

    // Verify the data of the rest of the model
    QStringList newData = getRows(model, QList<int>() << 1 << 2 << 3 << 4);
    QCOMPARE(newData.size(), oldData.size());
    QCOMPARE(newData.at(row), QString("ccc:1:20:20:"));
    for (int i = 0; i < row; ++i) {
        QCOMPARE(newData.at(i), oldData.at(i));
    }
    for (int i = row + 1; i < newData.size(); ++i) {
        QCOMPARE(newData.at(i), oldData.at(i));
    }
}

void tst_TrackerLiveAggregates::aggregatedResourceRelationshipChanged()
{
    // Note: this test deals only with changes in "nmm:performer" which is the
    // property connecting the aggreated resource (music piece) with the primary
    // resource (artist).
    QVERIFY(waitUntilModelFilled(liveQuery, 4));

    QStringList oldData = getRows(model, QList<int>() << 1 << 2 << 3 << 4);

    // Insert a resource
    QSignalSpy spy(model,
                   SIGNAL(dataChanged(const QModelIndex&, const QModelIndex&)));
    executeQuery(QSparqlQuery(QString(
                                  "delete { "
                                  "<song002a> nmm:performer <artist002> . "
                                  "} "
                                  "insert { "
                                  "<song002a> nmm:performer <artist001> . "
                                  "}"),
                              QSparqlQuery::InsertStatement));
    // BTW, why cannot a song have many performers?

    QTime timer;
    timer.start();
    while(timer.elapsed() < 5000 && spy.count() < 6)
        QTest::qWait(100);

    // Verify the modified rows
    QCOMPARE(spy.count(), 6);

    QSet<QPair<int, int> > coords;
    while (spy.count()) {
        QList<QVariant> parameters = spy.takeFirst();
        QModelIndex topLeft = parameters.at(0).value<QModelIndex>();
        QModelIndex bottomRight = parameters.at(1).value<QModelIndex>();
        QCOMPARE(topLeft.row(), bottomRight.row());
        QCOMPARE(topLeft.column(), bottomRight.column());
        coords.insert(qMakePair<int, int>(topLeft.row(), topLeft.column()));
    }
    QCOMPARE(coords.size(), 6);
    QVERIFY(coords.contains(qMakePair<int, int>(0, 2)));
    QVERIFY(coords.contains(qMakePair<int, int>(0, 3)));
    QVERIFY(coords.contains(qMakePair<int, int>(0, 4)));
    QVERIFY(coords.contains(qMakePair<int, int>(1, 2)));
    QVERIFY(coords.contains(qMakePair<int, int>(1, 3)));
    QVERIFY(coords.contains(qMakePair<int, int>(1, 4)));

    // Verify the data of the rest of the model
    QStringList newData = getRows(model, QList<int>() << 1 << 2 << 3 << 4);
    QCOMPARE(newData.size(), oldData.size());
    QCOMPARE(newData.at(0), QString("aaa:1:100:100:"));
    QCOMPARE(newData.at(1), QString("bbb:0:::"));
    for (int i = 2; i < newData.size(); ++i) {
        QCOMPARE(newData.at(i), oldData.at(i));
    }
}

void tst_TrackerLiveAggregates::aggregatedResourceChanged()
{
    // This test case tests changes in nfo:duration which is a property of the
    // aggregated resource (music piece) but has no connection to the primary
    // resource (artist).
    QVERIFY(waitUntilModelFilled(liveQuery, 4));

    QStringList oldData = getRows(model, QList<int>() << 1 << 2 << 3 << 4);

    // Modify a property of the aggregated resource.
    QSignalSpy spy(model,
                   SIGNAL(dataChanged(const QModelIndex&, const QModelIndex&)));
    executeQuery(QSparqlQuery(QString(
                                  "delete { "
                                  "<song003b> nfo:duration 20 . "
                                  "} "
                                  "insert { "
                                  "<song003b> nfo:duration 60 . "
                                  "}"),
                              QSparqlQuery::InsertStatement));
    // BTW, why cannot a song have many performers?

    QTime timer;
    timer.start();
    while(timer.elapsed() < 5000 && spy.count() < 2)
        QTest::qWait(100);

    QCOMPARE(spy.count(), 2);

    // Verify the modified rows
    QSet<QPair<int, int> > coords;
    while (spy.count()) {
        QList<QVariant> parameters = spy.takeFirst();
        QModelIndex topLeft = parameters.at(0).value<QModelIndex>();
        QModelIndex bottomRight = parameters.at(1).value<QModelIndex>();
        QCOMPARE(topLeft.row(), bottomRight.row());
        QCOMPARE(topLeft.column(), bottomRight.column());
        coords.insert(qMakePair<int, int>(topLeft.row(), topLeft.column()));
    }
    QCOMPARE(coords.size(), 2);
    QVERIFY(coords.contains(qMakePair<int, int>(2, 3)));
    QVERIFY(coords.contains(qMakePair<int, int>(2, 4)));

    // Verify the data of the rest of the model
    QStringList newData = getRows(model, QList<int>() << 1 << 2 << 3 << 4);
    QCOMPARE(newData.size(), oldData.size());
    QCOMPARE(newData.at(2), QString("ccc:2:50:60:"));
    int row = 2;
    for (int i = 0; i < row; ++i) {
        QCOMPARE(newData.at(i), oldData.at(i));
    }    for (int i = row + 1; i < newData.size(); ++i) {
        QCOMPARE(newData.at(i), oldData.at(i));
    }
}

QTEST_MAIN( tst_TrackerLiveAggregates )
#include "tst_live_aggregates.moc"
