/****************************************************************************
**
** Copyright (C) 2010-2011 Nokia Corporation and/or its subsidiary(-ies).
** All rights reserved.
** Contact: Nokia Corporation (ivan.frade@nokia.com)
**
** This file is part of QtSparqlTracker library.
**
** $QT_BEGIN_LICENSE:LGPL$
** GNU Lesser General Public License Usage
** This file may be used under the terms of the GNU Lesser General Public
** License version 2.1 as published by the Free Software Foundation and
** appearing in the file LICENSE.LGPL included in the packaging of this
** file. Please review the following information to ensure the GNU Lesser
** General Public License version 2.1 requirements will be met:
** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** In addition, as a special exception, Nokia gives you certain additional
** rights. These rights are described in the Nokia Qt LGPL Exception
** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU General
** Public License version 3.0 as published by the Free Software Foundation
** and appearing in the file LICENSE.GPL included in the packaging of this
** file. Please review the following information to ensure the GNU General
** Public License version 3.0 requirements will be met:
** http://www.gnu.org/copyleft/gpl.html.
**
** Other Usage
** Alternatively, this file may be used in accordance with the terms and
** conditions contained in a signed written agreement between you and Nokia.
**
** If you have questions regarding the use of this file, please contact
** Nokia at ivan.frade@nokia.com.
** $QT_END_LICENSE$
**
****************************************************************************/

#include "../utils/messagehandler.h"
#include "../utils/signalsrecorder.h"

#include "TrackerLiveQuery"

#include <QtTest/QtTest>
#include <QVariant>
#include <QtSparql/QtSparql>

class tst_TrackerLiveErrors : public QObject
{
    Q_OBJECT

public:
    tst_TrackerLiveErrors();
    virtual ~tst_TrackerLiveErrors();

public slots:
    void initTestCase();
    void cleanupTestCase();
    void init();
    void cleanup();

private slots:
    void cannotRunQueries();

private:
    MessageRecorder* msgRecorder;
};

tst_TrackerLiveErrors::tst_TrackerLiveErrors()
{
}

tst_TrackerLiveErrors::~tst_TrackerLiveErrors()
{
}

void tst_TrackerLiveErrors::initTestCase()
{
}

void tst_TrackerLiveErrors::cleanupTestCase()
{
}

void tst_TrackerLiveErrors::init()
{
    msgRecorder = new MessageRecorder;
    msgRecorder->addMsgTypeToRecord(QtWarningMsg);
}

void tst_TrackerLiveErrors::cleanup()
{
    delete msgRecorder;
    msgRecorder = 0;
}

void tst_TrackerLiveErrors::cannotRunQueries()
{
    QSparqlConnection bogusConn("BOGUS");
    // don't use the one created in init()
    TrackerLiveQuery brokenQuery("select 1 { }", 1,
                                 QList<int>() << 0,
                                 bogusConn);
    TrackerFullUpdater updater;
    updater.watchClass("xsd:boolean", QStringList());
    brokenQuery.addUpdater(updater);

    // test that we get the intialQueryFinished() signal
    SignalsRecorder signalRecorder;
    SignalList signalList;
    signalList.append(&brokenQuery, SIGNAL(initialQueryFinished()));
    signalRecorder.listen(signalList);

    QVERIFY( !msgRecorder->hasMsgsOfType(QtWarningMsg) );

    brokenQuery.start();

    QCOMPARE(signalRecorder.signalData(signalList).count(), 1);
    QVERIFY( msgRecorder->hasMsgsOfType(QtWarningMsg) );
}

QTEST_MAIN(tst_TrackerLiveErrors)
#include "tst_live_errors.moc"
