/****************************************************************************
**
** Copyright (C) 2010-2011 Nokia Corporation and/or its subsidiary(-ies).
** All rights reserved.
** Contact: Nokia Corporation (ivan.frade@nokia.com)
**
** This file is part of QtSparqlTracker library.
**
** $QT_BEGIN_LICENSE:LGPL$
** GNU Lesser General Public License Usage
** This file may be used under the terms of the GNU Lesser General Public
** License version 2.1 as published by the Free Software Foundation and
** appearing in the file LICENSE.LGPL included in the packaging of this
** file. Please review the following information to ensure the GNU Lesser
** General Public License version 2.1 requirements will be met:
** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** In addition, as a special exception, Nokia gives you certain additional
** rights. These rights are described in the Nokia Qt LGPL Exception
** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU General
** Public License version 3.0 as published by the Free Software Foundation
** and appearing in the file LICENSE.GPL included in the packaging of this
** file. Please review the following information to ensure the GNU General
** Public License version 3.0 requirements will be met:
** http://www.gnu.org/copyleft/gpl.html.
**
** Other Usage
** Alternatively, this file may be used in accordance with the terms and
** conditions contained in a signed written agreement between you and Nokia.
**
** If you have questions regarding the use of this file, please contact
** Nokia at ivan.frade@nokia.com.
** $QT_END_LICENSE$
**
****************************************************************************/

#include "TrackerLiveQuery"

#include <QtSparql>

#include <QCoreApplication>
#include <QDebug>
#include <QObject>
#include <QTimer>

#define RESOURCES_TO_INSERT 1000
#define CHANGE_INTERVAL 1500

// This adds data to Tracker / deletes the data
class DataModifier : public QObject
{
    Q_OBJECT
public:
    DataModifier(QSparqlConnection* c)
        : conn(c), i(0)
    {
    }
public slots:
    void change()
    {
        static bool inserting = true;
        if (inserting) {
            doInsert();
            ++i;
            if (i >= RESOURCES_TO_INSERT) {
                inserting = false;
                --i;
            }
        }
        else {
            doDelete();
            --i;
            if (i < 0)
                QCoreApplication::quit();
        }
    }
private:
    QSparqlConnection* conn;
    int i;
    void doInsert()
    {
        qDebug() << "inserting" << i;
        QString query = QString("insert { <mp%1> a nmm:MusicPiece ; "
                                "nie:isLogicalPartOf <tracker-live-tests> ; "
                                "nfo:duration %1 .}").arg(i);
        QSparqlResult* r = conn->exec(QSparqlQuery(query, QSparqlQuery::InsertStatement));
        r->waitForFinished();
        delete r;
    }
    void doDelete()
    {
        qDebug() << "deleting" << i;
        QString query = QString("delete { <mp%1> a rdfs:Resource .}").arg(i);
        QSparqlResult* r = conn->exec(QSparqlQuery(query, QSparqlQuery::DeleteStatement));
        r->waitForFinished();
        delete r;
    }
};

int main(int argc, char** argv)
{
    QCoreApplication app(argc, argv);

    QSparqlConnection conn("QTRACKER_DIRECT");
    DataModifier mod(&conn);

    QTimer timer;
    timer.setInterval(CHANGE_INTERVAL);
    timer.setSingleShot(false);
    QObject::connect(&timer, SIGNAL(timeout()),
                     &mod, SLOT(change()));

    QString query = "select tracker:id(?mp) ?mp ?l "
        "{ ?mp a nmm:MusicPiece ; "
        "nie:isLogicalPartOf <tracker-live-tests> ; "
        "nfo:duration ?l . %1 }"
        " order by ?l";

    TrackerLiveQuery liveQuery(query.arg(""), 3, QList<int>() << 0, conn);
    TrackerPartialUpdater up(query.arg("%FILTER"));
    up.watchClass("nmm:MusicPiece",
                  QStringList() << "nfo:duration" << "rdf:type",
                  "tracker:id(?mp) in %LIST", TrackerPartialUpdater::Subject, 0);
    liveQuery.addUpdater(up);
    liveQuery.setCollationColumns(QList<TrackerLiveQuery::CollationColumn>() <<
                                  TrackerLiveQuery::CollationColumn(2, QVariant::Int));

    timer.start();
    return app.exec();
}

#include "tst_live_memory.moc"
