/****************************************************************************
**
** Copyright (C) 2010-2011 Nokia Corporation and/or its subsidiary(-ies).
** All rights reserved.
** Contact: Nokia Corporation (ivan.frade@nokia.com)
**
** This file is part of QtSparqlTracker library.
**
** $QT_BEGIN_LICENSE:LGPL$
** GNU Lesser General Public License Usage
** This file may be used under the terms of the GNU Lesser General Public
** License version 2.1 as published by the Free Software Foundation and
** appearing in the file LICENSE.LGPL included in the packaging of this
** file. Please review the following information to ensure the GNU Lesser
** General Public License version 2.1 requirements will be met:
** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** In addition, as a special exception, Nokia gives you certain additional
** rights. These rights are described in the Nokia Qt LGPL Exception
** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU General
** Public License version 3.0 as published by the Free Software Foundation
** and appearing in the file LICENSE.GPL included in the packaging of this
** file. Please review the following information to ensure the GNU General
** Public License version 3.0 requirements will be met:
** http://www.gnu.org/copyleft/gpl.html.
**
** Other Usage
** Alternatively, this file may be used in accordance with the terms and
** conditions contained in a signed written agreement between you and Nokia.
**
** If you have questions regarding the use of this file, please contact
** Nokia at ivan.frade@nokia.com.
** $QT_END_LICENSE$
**
****************************************************************************/

#include "../utils/livetesthelpers.h"
#include "../utils/updatewaiter.h"

#include "TrackerLiveQuery"

#include <QtTest/QtTest>
#include <QtSparql/QtSparql>

//const QString qtest(qTableName( "qtest", __FILE__ )); // FIXME: what's this

//TESTED_FILES=

/*

This test contains a multi-valued live model:
column 0: tracker:id of a feed channel
column 1: title of the feed channel
column 2: tracker:id of a feed message belonging to the channel (one channel can
have many)
column 3: subject of the message

*/

class tst_TrackerLiveMultiValue : public QObject
{
    Q_OBJECT

public:
    tst_TrackerLiveMultiValue();
    virtual ~tst_TrackerLiveMultiValue();

public slots:
    void initTestCase();
    void cleanupTestCase();
    void init();
    void cleanup();

private slots:
    void initialModel();

    void primaryResourceAdded();
    void primaryResourceRemoved();
    void primaryResourceChanged();

    void secondaryResourceAdded();
    void secondaryResourceRemoved();
    void secondaryResourceChanged();

    void primaryAndSecondaryResourceAdded();

private:
    // Object under test
    TrackerLiveQuery* liveQuery;
    QSparqlConnection* conn;
    QAbstractItemModel* model;
};

Q_DECLARE_METATYPE(QModelIndex)

tst_TrackerLiveMultiValue::tst_TrackerLiveMultiValue()
: liveQuery(0), conn(0), model(0)
{
}

tst_TrackerLiveMultiValue::~tst_TrackerLiveMultiValue()
{
}

void tst_TrackerLiveMultiValue::initTestCase()
{
    qRegisterMetaType<QModelIndex>("QModelIndex");
    // Clean up leftover data from previous test run
    cleanup();
}

void tst_TrackerLiveMultiValue::cleanupTestCase()
{
}

void tst_TrackerLiveMultiValue::init()
{
    // Insert initial test data
    UpdateWaiter waiterFeedCh(QString("http://www.tracker-project.org/temp/mfo#FeedChannel"), 6);
    UpdateWaiter waiterFeedM(QString("http://www.tracker-project.org/temp/mfo#FeedMessage"), 16);
    executeQuery(QSparqlQuery(
                     "insert { "
                     "<tracker-live-tests> a nie:InformationElement . "
                     "<channel001> a mfo:FeedChannel ; "
                     "  nie:isLogicalPartOf <tracker-live-tests> ; "
                     "  nie:title \"channel 1\" . "
                     "<channel002> a mfo:FeedChannel ; "
                     "  nie:isLogicalPartOf <tracker-live-tests> ; "
                     "  nie:title \"channel 2\" . "
                     "<feed001> a mfo:FeedMessage ; "
                     "  nie:isLogicalPartOf <tracker-live-tests> ; "
                     "  nie:isLogicalPartOf <channel001> ; "
                     "  nmo:messageSubject \"feed 1\" . "
                     "<feed002> a mfo:FeedMessage ; "
                     "  nie:isLogicalPartOf <tracker-live-tests> ; "
                     "  nie:isLogicalPartOf <channel001> ; "
                     "  nie:isLogicalPartOf <channel002> ; "
                     "  nmo:messageSubject \"feed 2\" ."
                     "<feed003> a mfo:FeedMessage ; "
                     "  nie:isLogicalPartOf <tracker-live-tests> ; "
                     "  nie:isLogicalPartOf <channel002> ; "
                     "  nmo:messageSubject \"feed 3\" . "
                     "<feed004> a mfo:FeedMessage ; "
                     "  nie:isLogicalPartOf <tracker-live-tests> ; "
                     "  nmo:messageSubject \"feed 4\" . "
                     "}",
                     QSparqlQuery::InsertStatement),
                 false);
    // Wait until all inserted data is in tracker
    QVERIFY(waiterFeedCh.waitForAllUpdated());
    QVERIFY(waiterFeedM.waitForAllUpdated());

    // Create the object under test
    QString query("select tracker:id(?c) ?ct tracker:id(?m) ?s "
                  "{ "
                  "?c a mfo:FeedChannel ; "
                  "nie:isLogicalPartOf <tracker-live-tests> ;"
                  "nie:title ?ct . "
                  "?m a mfo:FeedMessage ; "
                  "nmo:messageSubject ?s ; "
                  "nie:isLogicalPartOf ?c . "
                  "%1"
                  "}");
    conn = new QSparqlConnection("QTRACKER_DIRECT");
    // Note: Using the deprecated TrackerLiveQuery constructor here.  Start
    // using the new constructor when the deprecated one is removed.
    liveQuery = new TrackerLiveQuery(query.arg(""), 4, *conn);

    TrackerPartialUpdater up(query.arg("%FILTER"));
    up.watchClass("mfo:FeedChannel", QStringList(),
                  "tracker:id(?c) in %LIST", TrackerPartialUpdater::Subject, 0);

    up.watchClass("mfo:FeedMessage", QStringList(),
                  "tracker:id(?m) in %LIST", TrackerPartialUpdater::Subject, 2);

    liveQuery->addUpdater(up);

    liveQuery->setIdentityColumns(QList<int>() << 0 << 2);
    liveQuery->start();

    // store this for convenience
    model = liveQuery->model();
}

void tst_TrackerLiveMultiValue::cleanup()
{
    // Delete the tested object
    delete liveQuery;
    liveQuery = 0;
    model = 0;
    delete conn;
    conn = 0;

    // Clean up test data
    executeQuery(QSparqlQuery(
                     "delete { "
                     "?u a rdfs:Resource . } "
                     "where { "
                     "?u nie:isLogicalPartOf <tracker-live-tests> . "
                     "} "
                     "delete {"
                     "<tracker-live-tests> a nie:InformationElement . "
                     "}",
                     QSparqlQuery::DeleteStatement));
}

void tst_TrackerLiveMultiValue::initialModel()
{
    // Ignore the return value; we'll do the checks here
    waitUntilModelFilled(liveQuery, 4);

    QCOMPARE(model->rowCount(), 4);
    QCOMPARE(model->columnCount(), 4);

    // Verify the data but don't care about ordering.
    QStringList data = getRows(model, QList<int>() << 1 << 3);
    QVERIFY(data.contains(QString("channel 1:feed 1:")));
    QVERIFY(data.contains(QString("channel 1:feed 2:")));
    QVERIFY(data.contains(QString("channel 2:feed 2:")));
    QVERIFY(data.contains(QString("channel 2:feed 3:")));
}

void tst_TrackerLiveMultiValue::primaryResourceAdded()
{
    QVERIFY(waitUntilModelFilled(liveQuery, 4));

    // Insert a primary resource (channel), associate it with a already-existing
    // secondary resource (feed message).
    QSignalSpy spy(model, SIGNAL(rowsInserted(const QModelIndex&, int, int)));
    executeQuery(QSparqlQuery(
                     "insert {"
                     "<addedchannel001> a mfo:FeedChannel ; "
                     "  nie:isLogicalPartOf <tracker-live-tests> ; "
                     "  nie:title \"added channel\" . "
                     "<feed004> nie:isLogicalPartOf <addedchannel001>. }",
                     QSparqlQuery::InsertStatement));
    QTime timer;
    timer.start();
    while(timer.elapsed() < 5000 && spy.count() < 1)
        QTest::qWait(100);

    // Verify the inserted row but don't care about ordering
    QCOMPARE(spy.count(), 1);
    QList<QVariant> parameters = spy.takeFirst();
    QModelIndex ix = parameters.at(0).value<QModelIndex>();
    int start = parameters.at(1).value<int>();
    int end = parameters.at(2).value<int>();

    QCOMPARE(ix, QModelIndex());
    QCOMPARE(start, end); // only 1 row inserted

    QCOMPARE(model->data(model->index(start, 1)).toString(),
             QString("added channel"));
    QCOMPARE(model->data(model->index(start, 3)).toString(),
             QString("feed 4"));
}

void tst_TrackerLiveMultiValue::primaryResourceRemoved()
{
    QVERIFY(waitUntilModelFilled(liveQuery, 4));

    // Delete a resource
    executeQuery(QSparqlQuery(
                     "delete {"
                     "<channel001> a rdfs:Resource . }",
                     QSparqlQuery::DeleteStatement));
    QTime timer;
    timer.start();
    while(timer.elapsed() < 5000 && model->rowCount() > 2)
        QTest::qWait(100);

    // The test just verifies that the contents of the model are correct. The
    // correctness of rowsRemoved signals are not (yet) verified.
    QCOMPARE(model->rowCount(), 2);

    QStringList data = getRows(model, QList<int>() << 1 << 3);
    QVERIFY(data.contains(QString("channel 2:feed 2:")));
    QVERIFY(data.contains(QString("channel 2:feed 3:")));
}

void tst_TrackerLiveMultiValue::primaryResourceChanged()
{
    QVERIFY(waitUntilModelFilled(liveQuery, 4));

    // Find the affected rows (their indices won't change, so we can safely
    // store them).
    QSet<int> rows;
    for (int i = 0; i < model->rowCount(); ++i)
        if (model->data(model->index(i, 1)).toString() == QString("channel 2"))
            rows.insert(i);

    // Modify a property of a primary resource (channel)
    QSignalSpy spy(model,
                   SIGNAL(dataChanged(const QModelIndex&, const QModelIndex&)));
    executeQuery(QSparqlQuery(
                     "delete { "
                     "<channel002> nie:title \"channel 2\" . "
                     "}"
                     "insert { "
                     "<channel002> nie:title \"channel changed\" . "
                     "}",
                     QSparqlQuery::DeleteStatement));
    QTime timer;
    timer.start();
    // Note: this test assumes we get one dataChanged signal per each row.
    while(timer.elapsed() < 10000 && spy.count() < rows.size())
        QTest::qWait(100);

    // Verify the row got modified
    QCOMPARE(spy.count(), rows.size());
    for (int i = 0; i < spy.count(); ++i) {
        QList<QVariant> parameters = spy.at(i);
        QModelIndex topLeft = parameters.at(0).value<QModelIndex>();
        QModelIndex bottomRight = parameters.at(1).value<QModelIndex>();

        QCOMPARE(topLeft.row(), bottomRight.row());
        QVERIFY(rows.contains(topLeft.row()));
        rows.remove(topLeft.row());

        QCOMPARE(topLeft.column(), 1);
        QCOMPARE(bottomRight.column(), 1);

        QCOMPARE(model->data(model->index(topLeft.row(), 1)).toString(),
                 QString("channel changed"));
    }
    QVERIFY(rows.size() == 0);
}

void tst_TrackerLiveMultiValue::secondaryResourceAdded()
{
    QVERIFY(waitUntilModelFilled(liveQuery, 4));

    // Insert a secondary resource (contact), and associate it with an
    // alredy-existing primary resource (e-mail)
    QSignalSpy spy(model, SIGNAL(rowsInserted(const QModelIndex&, int, int)));
    executeQuery(QSparqlQuery(
                     "insert {"
                     "<addedfeed001> a mfo:FeedMessage ; "
                     "  nie:isLogicalPartOf <tracker-live-tests> ; "
                     "  nmo:messageSubject \"added feed 1\" ;"
                     "  nie:isLogicalPartOf <channel001> . }",
                     QSparqlQuery::InsertStatement));
    QTime timer;
    timer.start();
    while(timer.elapsed() < 5000 && spy.count() < 1)
        QTest::qWait(100);

    // Verify the inserted row but don't care about ordering
    QCOMPARE(spy.count(), 1);
    QList<QVariant> parameters = spy.at(0);
    QModelIndex ix = parameters.at(0).value<QModelIndex>();
    int start = parameters.at(1).value<int>();
    int end = parameters.at(2).value<int>();

    QCOMPARE(ix, QModelIndex());
    QCOMPARE(start, end); // only 1 row inserted

    QCOMPARE(model->data(model->index(start, 1)).toString(),
             QString("channel 1"));
    QCOMPARE(model->data(model->index(start, 3)).toString(),
             QString("added feed 1"));

    // Verify the data
    QCOMPARE(model->rowCount(), 5);
    QStringList data = getRows(model, QList<int>() << 1 << 3);
    QVERIFY(data.contains(QString("channel 1:feed 1:")));
    QVERIFY(data.contains(QString("channel 1:feed 2:")));
    QVERIFY(data.contains(QString("channel 1:added feed 1:")));
    QVERIFY(data.contains(QString("channel 2:feed 2:")));
    QVERIFY(data.contains(QString("channel 2:feed 3:")));
}

void tst_TrackerLiveMultiValue::secondaryResourceRemoved()
{
    QVERIFY(waitUntilModelFilled(liveQuery, 4));

    // Delete a secondary resource (message) so that its primary resource will
    // remain in the model (it has other secondary resources).
    executeQuery(QSparqlQuery(
                     "delete {"
                     "<feed003> a rdfs:Resource . }",
                     QSparqlQuery::DeleteStatement));
    QTime timer;
    timer.start();
    while(timer.elapsed() < 5000 && model->rowCount() > 3)
        QTest::qWait(100);

    // The test just verifies that the contents of the model are correct. The
    // correctness of rowsRemoved signals are not (yet) verified.
    QCOMPARE(model->rowCount(), 3);

    QStringList data = getRows(model, QList<int>() << 1 << 3);
    QVERIFY(data.contains(QString("channel 1:feed 1:")));
    QVERIFY(data.contains(QString("channel 1:feed 2:")));
    QVERIFY(data.contains(QString("channel 2:feed 2:")));

    // Delete another secondary resource (message) so that its primary resource
    // will also be deleted (it has no secondary resources left).
    executeQuery(QSparqlQuery(
                     "delete {"
                     "<feed002> a rdfs:Resource . }",
                     QSparqlQuery::DeleteStatement));
    timer.start();
    while(timer.elapsed() < 5000 && model->rowCount() > 1)
        QTest::qWait(100);

    // The test just verifies that the contents of the model are correct. The
    // correctness of rowsRemoved signals are not (yet) verified.
    QCOMPARE(model->rowCount(), 1);

    data = getRows(model, QList<int>() << 1 << 3);
    QVERIFY(data.contains(QString("channel 1:feed 1:")));
}

void tst_TrackerLiveMultiValue::secondaryResourceChanged()
{
    QVERIFY(waitUntilModelFilled(liveQuery, 4));

    // Find the affected rows (their indices won't change, so we can safely
    // store them).
    int row;
    for (row = 0; row < model->rowCount(); ++row)
        if (model->data(model->index(row, 3)).toString() == QString("feed 3"))
            break;

    // Modify a property of a secondary resource (message)
    QSignalSpy spy(model,
                   SIGNAL(dataChanged(const QModelIndex&, const QModelIndex&)));
    executeQuery(QSparqlQuery(
                     "delete { "
                     "<feed003> nmo:messageSubject \"feed 3\" . "
                     "}"
                     "insert { "
                     "<feed003> nmo:messageSubject \"feed changed\" . "
                     "}",
                     QSparqlQuery::DeleteStatement));
    QTime timer;
    timer.start();
    while(timer.elapsed() < 10000 && spy.count() < 1)
        QTest::qWait(100);

    // Verify the row got modified
    QCOMPARE(spy.count(), 1);
    QList<QVariant> parameters = spy.at(0);
    QModelIndex topLeft = parameters.at(0).value<QModelIndex>();
    QModelIndex bottomRight = parameters.at(1).value<QModelIndex>();

    QCOMPARE(topLeft.row(), bottomRight.row()); // only one row was changed
    QCOMPARE(topLeft.row(), row);

    QCOMPARE(topLeft.column(), 3);
    QCOMPARE(bottomRight.column(), 3);

    QCOMPARE(model->data(model->index(topLeft.row(), 3)).toString(),
             QString("feed changed"));

    // Verify the data
    QCOMPARE(model->rowCount(), 4);
    QStringList data = getRows(model, QList<int>() << 1 << 3);
    QVERIFY(data.contains(QString("channel 1:feed 1:")));
    QVERIFY(data.contains(QString("channel 1:feed 2:")));
    QVERIFY(data.contains(QString("channel 2:feed 2:")));
    QVERIFY(data.contains(QString("channel 2:feed changed:")));
}

void tst_TrackerLiveMultiValue::primaryAndSecondaryResourceAdded()
{
    QVERIFY(waitUntilModelFilled(liveQuery, 4));

    // Insert a primary resource (channel), and a secondary resource (message),
    // associate the newly-added resources.
    QSignalSpy spy(model, SIGNAL(rowsInserted(const QModelIndex&, int, int)));
    executeQuery(QSparqlQuery(
                     "insert {"
                     "<addedchannel001> a mfo:FeedChannel ; "
                     "  nie:isLogicalPartOf <tracker-live-tests> ; "
                     "  nie:title \"added channel\" . "
                     "<addedfeed001> a mfo:FeedMessage ; "
                     "  nie:isLogicalPartOf <tracker-live-tests> ; "
                     "  nmo:messageSubject \"added feed 1\" ; "
                     "  nie:isLogicalPartOf <addedchannel001>. }",
                     QSparqlQuery::InsertStatement));
    QTime timer;
    timer.start();
    while(timer.elapsed() < 5000 && spy.count() < 1)
        QTest::qWait(100);

    // Verify the inserted row but don't care about ordering
    QCOMPARE(spy.count(), 1);
    QList<QVariant> parameters = spy.takeFirst();
    QModelIndex ix = parameters.at(0).value<QModelIndex>();
    int start = parameters.at(1).value<int>();
    int end = parameters.at(2).value<int>();

    QCOMPARE(ix, QModelIndex());
    QCOMPARE(start, end); // only 1 row inserted

    QCOMPARE(model->data(model->index(start, 1)).toString(),
             QString("added channel"));
    QCOMPARE(model->data(model->index(start, 3)).toString(),
             QString("added feed 1"));
}


QTEST_MAIN( tst_TrackerLiveMultiValue )
#include "tst_live_multivalue.moc"
