/****************************************************************************
**
** Copyright (C) 2010-2011 Nokia Corporation and/or its subsidiary(-ies).
** All rights reserved.
** Contact: Nokia Corporation (ivan.frade@nokia.com)
**
** This file is part of QtSparqlTracker library.
**
** $QT_BEGIN_LICENSE:LGPL$
** GNU Lesser General Public License Usage
** This file may be used under the terms of the GNU Lesser General Public
** License version 2.1 as published by the Free Software Foundation and
** appearing in the file LICENSE.LGPL included in the packaging of this
** file. Please review the following information to ensure the GNU Lesser
** General Public License version 2.1 requirements will be met:
** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** In addition, as a special exception, Nokia gives you certain additional
** rights. These rights are described in the Nokia Qt LGPL Exception
** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU General
** Public License version 3.0 as published by the Free Software Foundation
** and appearing in the file LICENSE.GPL included in the packaging of this
** file. Please review the following information to ensure the GNU General
** Public License version 3.0 requirements will be met:
** http://www.gnu.org/copyleft/gpl.html.
**
** Other Usage
** Alternatively, this file may be used in accordance with the terms and
** conditions contained in a signed written agreement between you and Nokia.
**
** If you have questions regarding the use of this file, please contact
** Nokia at ivan.frade@nokia.com.
** $QT_END_LICENSE$
**
****************************************************************************/

#include "TrackerChangeNotifier"

#include <QtTest/QtTest>
#include <QtSparql/QtSparql>

//const QString qtest(qTableName( "qtest", __FILE__ )); // FIXME: what's this

//TESTED_FILES=

class Receiver : public QObject
{
    Q_OBJECT

public slots:
    void changed(QList<TrackerChangeNotifier::Quad> d, QList<TrackerChangeNotifier::Quad> i);
public:
    QList<TrackerChangeNotifier::Quad> deletes;
    QList<TrackerChangeNotifier::Quad> inserts;
};

void Receiver::changed(QList<TrackerChangeNotifier::Quad> d, QList<TrackerChangeNotifier::Quad> i)
{
    deletes.append(d);
    inserts.append(i);
}

class tst_TrackerSignals : public QObject
{
    Q_OBJECT

public:
    tst_TrackerSignals();
    virtual ~tst_TrackerSignals();

private:
    void insertContacts(QStringList uris, QStringList names);
    void removeContacts(QStringList uris);

public slots:
    void initTestCase();
    void cleanupTestCase();
    void init();
    void cleanup();

private slots:
    void contact_added();
    void contact_deleted();
    void recreateChangeNotifier();
    void insertsBuffered();
    void removesBuffered();
    void switchBuffer();

private:
    QString className;
    int typeId, personContactId, nameGivenId;
};

tst_TrackerSignals::tst_TrackerSignals()
{
}

tst_TrackerSignals::~tst_TrackerSignals()
{
}

void tst_TrackerSignals::insertContacts(QStringList uris, QStringList names)
{

    QSparqlConnection conn("QTRACKER_DIRECT");
    for(int i=0; i<names.size(); i++)
    {
        QSparqlQuery q(QString("insert { <%2> a nco:PersonContact ;"
        "nie:isLogicalPartOf <libqtsparql-tracker-tests> ;"
        "nco:nameGiven \"%1\" .}").arg(names.at(i)).arg(uris.at(i)), QSparqlQuery::InsertStatement);
        QSparqlResult* r = conn.exec(q);
        QVERIFY(r != 0);
        QVERIFY(!r->hasError());
        r->waitForFinished();
        delete r;
    }
}

void tst_TrackerSignals::removeContacts(QStringList uris)
{

    QSparqlConnection conn("QTRACKER_DIRECT");
    for(int i=0; i<uris.size(); i++)
    {
        QSparqlQuery q(QString("delete { <%1> nie:isLogicalPartOf ?lp ;"
        "nco:nameGiven ?n ;"
        "a nco:PersonContact .} where"
        "{ <%1> nie:isLogicalPartOf ?lp ;"
        "nco:nameGiven ?n .}").arg(uris.at(i)), QSparqlQuery::DeleteStatement);
        QSparqlResult* r = conn.exec(q);
        QVERIFY(r != 0);
        QVERIFY(!r->hasError());
        r->waitForFinished();
        delete r;
    }
}

void tst_TrackerSignals::initTestCase()
{
    // Query: what's the current name of the contacts class?
    QSparqlConnection conn("QTRACKER_DIRECT");
    QSparqlQuery q("select tracker:uri(tracker:id(nco:PersonContact)) "
                   "tracker:id(rdf:type) "
                   "tracker:id(nco:PersonContact) "
                   "tracker:id(nco:nameGiven) "
                   "{}");
    QSparqlResult* r = conn.exec(q);
    r->waitForFinished();
    if (r->hasError()) {
        qWarning() << r->lastError().message();
        qFatal("Initial query: error");
    }
    if (r->next() && r->current().count() == 4) {
        className = r->value(0).toString();
        typeId = r->value(1).toInt();
        personContactId = r->value(2).toInt();
        nameGivenId = r->value(3).toInt();
        // Useful prints for debugging
        //qDebug() << "PersonContact" << personContactId;
        //qDebug() << "type" << typeId;
        //qDebug() << "nameGiven" << nameGivenId;
    }
    else
        qFatal("Initial query: not enough data");
    delete r;
    // For QSignalSpy
    qRegisterMetaType<QList<TrackerChangeNotifier::Quad> >("QList<TrackerChangeNotifier::Quad>");

    // clean up possible leftover data from the previous leftrun.
    cleanup();
}

void tst_TrackerSignals::cleanupTestCase()
{
}

void tst_TrackerSignals::init()
{
}

void tst_TrackerSignals::cleanup()
{
    QSparqlQuery q("delete { ?u a rdfs:Resource . }"
                   "where {?u nie:isLogicalPartOf <libqtsparql-tracker-tests> . }"
                   "delete {"
                   "<tracker-live-tests> a nie:InformationElement . "
                   "}", QSparqlQuery::DeleteStatement);
    QSparqlConnection conn("QTRACKER_DIRECT");
    QSparqlResult* r = conn.exec(q);
    r->waitForFinished();
    delete r;
    // wait long enough so that the resulting GraphChanged doesn't affect future
    // test cases.
    QTest::qWait(2000);
}

// For QSignalSpy
Q_DECLARE_METATYPE(QList<TrackerChangeNotifier::Quad>)

void tst_TrackerSignals::contact_added()
{
    TrackerChangeNotifier notifier(className);

    QSignalSpy spy(&notifier,
                   SIGNAL(changed(QList<TrackerChangeNotifier::Quad>,
                                  QList<TrackerChangeNotifier::Quad>)));

    // TODO: also read the parameters from the spy.
    Receiver receiver;
    QObject::connect(&notifier,
                     SIGNAL(changed(QList<TrackerChangeNotifier::Quad>,
                                    QList<TrackerChangeNotifier::Quad>)),
                     &receiver,
                     SLOT(changed(QList<TrackerChangeNotifier::Quad>,
                                  QList<TrackerChangeNotifier::Quad>)));

    // Now do an insert...
    QSparqlQuery q("insert { <added.uri> a nco:PersonContact ;"
                   "nie:isLogicalPartOf <libqtsparql-tracker-tests> ;"
                   "nco:nameGiven \"temp.name\" .}", QSparqlQuery::InsertStatement);
    QSparqlConnection conn("QTRACKER_DIRECT");
    QSparqlResult* r = conn.exec(q);
    QVERIFY(r != 0);
    QVERIFY(!r->hasError());
    r->waitForFinished();
    QVERIFY(!r->hasError());
    delete r;
    r = 0;

    // And process the events so that
    // 1) tracker sends the signal through D-Bus
    // 2) TrackerChangeNotifier gets the signal and sends it to the spy

    QTime timer;
    timer.start();
    while(timer.elapsed() < 5000 && !spy.count())
        QCoreApplication::processEvents();

    QCOMPARE(spy.count(), 1);

    QVERIFY(receiver.deletes.size() == 0);
    QVERIFY(receiver.inserts.size() == 3);

    // The triples that are inserted:
    // newid rdf:type nco:PersonContact
    // newid nie:isLogicalPartOf testcontext
    // newid nameGiven 0

    // The graph is the default graph
    QCOMPARE(receiver.inserts[0].graph, 0);
    QCOMPARE(receiver.inserts[1].graph, 0);
    QCOMPARE(receiver.inserts[2].graph, 0);

    // The newid is the same for all rows
    QCOMPARE(receiver.inserts[0].subject, receiver.inserts[1].subject);
    QCOMPARE(receiver.inserts[0].subject, receiver.inserts[2].subject);

    //qDebug() << receiver.inserts;

    bool typeFound = false;
    bool nameFound = false;
    for (int i=0; i<3; ++i) {
        if (receiver.inserts[i].predicate == typeId && receiver.inserts[i].object == personContactId)
            typeFound = true;
        else if (receiver.inserts[i].predicate == nameGivenId && receiver.inserts[i].object == 0)
            nameFound = true;
    }
    QVERIFY(typeFound);
    QVERIFY(nameFound);
}

void tst_TrackerSignals::contact_deleted()
{
    // First insert some data so that we can delete it. We need to make sure
    // that it was properly inserted (and the appropriate signal was sent)
    // before starting the real test.
    TrackerChangeNotifier dummyNotifier(className);
    QSignalSpy dummySpy(&dummyNotifier,
                        SIGNAL(changed(QList<TrackerChangeNotifier::Quad>,
                                       QList<TrackerChangeNotifier::Quad>)));

    QSparqlQuery q("insert { <uri.to.delete> a nco:PersonContact ;"
                   "nie:isLogicalPartOf <libqtsparql-tracker-tests> ;"
                   "nco:nameGiven \"temp.name\" .}", QSparqlQuery::InsertStatement);
    QSparqlConnection conn("QTRACKER_DIRECT");
    QSparqlResult* r = conn.exec(q);
    QVERIFY(r != 0);
    QVERIFY(!r->hasError());
    r->waitForFinished();
    QVERIFY(!r->hasError());
    delete r;
    r = 0;

    QTime timer;
    timer.start();
    while(timer.elapsed() < 5000 && !dummySpy.count())
        QCoreApplication::processEvents();

    // Then start listening to changes
    TrackerChangeNotifier notifier(className);
    QSignalSpy spy(&notifier,
                   SIGNAL(changed(QList<TrackerChangeNotifier::Quad>,
                                  QList<TrackerChangeNotifier::Quad>)));

    // TODO: also read the parameters from the spy.
    Receiver receiver;
    QObject::connect(&notifier,
                     SIGNAL(changed(QList<TrackerChangeNotifier::Quad>,
                                    QList<TrackerChangeNotifier::Quad>)),
                     &receiver,
                     SLOT(changed(QList<TrackerChangeNotifier::Quad>,
                                  QList<TrackerChangeNotifier::Quad>)));

    // Then do the deletion. Delete in this order so that we will get all the
    // triples in the delete array. If we delete "<uri.to.delete> a
    // nco:PersonContact" first, we'll get only that triple.
    QSparqlQuery q2("delete { <uri.to.delete> nie:isLogicalPartOf ?lp ;"
                   "nco:nameGiven ?n ;"
                    "a nco:PersonContact .} where"
                   "{ <uri.to.delete> nie:isLogicalPartOf ?lp ;"
                   "nco:nameGiven ?n .}", QSparqlQuery::DeleteStatement);

    r = conn.exec(q2);
    QVERIFY(r != 0);
    QVERIFY(!r->hasError());
    r->waitForFinished();
    QVERIFY(!r->hasError());
    delete r;

    // And process the events so that
    // 1) tracker sends the signal through D-Bus
    // 2) TrackerChangeNotifier gets the signal and sends it to the spy

    timer.start();
    while(timer.elapsed() < 5000 && !spy.count())
        QCoreApplication::processEvents();

    QCOMPARE(spy.count(), 1);

    //qDebug() << receiver.inserts;
    //qDebug() << receiver.deletes;

    QVERIFY(receiver.deletes.size() == 3);
    QVERIFY(receiver.inserts.size() == 0);


    // The triples that are deleted:
    // someid rdf:type nco:PersonContact
    // someid nie:isLogicalPartOf testcontext
    // someid nameGiven 0

    // The graph is the default graph
    QCOMPARE(receiver.deletes[0].graph, 0);
    QCOMPARE(receiver.deletes[1].graph, 0);
    QCOMPARE(receiver.deletes[2].graph, 0);

    // The someid is the same for all rows
    QCOMPARE(receiver.deletes[0].subject, receiver.deletes[1].subject);
    QCOMPARE(receiver.deletes[0].subject, receiver.deletes[2].subject);

    bool typeFound = false;
    bool nameFound = false;
    for (int i=0; i<3; ++i) {
        if (receiver.deletes[i].predicate == typeId && receiver.deletes[i].object == personContactId)
            typeFound = true;
        else if (receiver.deletes[i].predicate == nameGivenId && receiver.deletes[i].object == 0)
            nameFound = true;
    }
    QVERIFY(typeFound);
    QVERIFY(nameFound);
}

void tst_TrackerSignals::recreateChangeNotifier()
{
    {
        // create a TrackerChangeNotifier and check that it gets some signals.
        TrackerChangeNotifier notifier(className);

        QSignalSpy spy(&notifier,
                       SIGNAL(changed(QList<TrackerChangeNotifier::Quad>,
                                      QList<TrackerChangeNotifier::Quad>)));

        // Now do an insert...
        QSparqlQuery q("insert { <added.uri> a nco:PersonContact ;"
                       "nie:isLogicalPartOf <libqtsparql-tracker-tests> ;"
                       "nco:nameGiven \"temp.name\" .}", QSparqlQuery::InsertStatement);
        QSparqlConnection conn("QTRACKER_DIRECT");
        QSparqlResult* r = conn.exec(q);
        r->waitForFinished();
        QVERIFY(!r->hasError());
        delete r;
        r = 0;

        // And process the events so that
        // 1) tracker sends the signal through D-Bus
        // 2) TrackerChangeNotifier gets the signal and sends it to the spy

        QTime timer;
        timer.start();
        while(timer.elapsed() < 5000 && !spy.count())
            QCoreApplication::processEvents();

        QCOMPARE(spy.count(), 1);
        // notifier goes out of scope
    }
    // Do it again. This is not inside a for loop since 1) the added uri is
    // different, 2) to get more useful line numbers for failures.
    {
        // create a TrackerChangeNotifier and check that it gets some signals.
        TrackerChangeNotifier notifier(className);

        QSignalSpy spy(&notifier,
                       SIGNAL(changed(QList<TrackerChangeNotifier::Quad>,
                                      QList<TrackerChangeNotifier::Quad>)));

        // Now do an insert...
        QSparqlQuery q("insert { <added.uri2> a nco:PersonContact ;"
                       "nie:isLogicalPartOf <libqtsparql-tracker-tests> ;"
                       "nco:nameGiven \"temp.name\" .}", QSparqlQuery::InsertStatement);
        QSparqlConnection conn("QTRACKER_DIRECT");
        QSparqlResult* r = conn.exec(q);
        r->waitForFinished();
        QVERIFY(!r->hasError());
        delete r;
        r = 0;

        // And process the events so that
        // 1) tracker sends the signal through D-Bus
        // 2) TrackerChangeNotifier gets the signal and sends it to the spy

        QTime timer;
        timer.start();
        while(timer.elapsed() < 5000 && !spy.count())
            QCoreApplication::processEvents();

        QCOMPARE(spy.count(), 1);
        // notifier goes out of scope
    }
}

void tst_TrackerSignals::insertsBuffered()
{
    TrackerChangeNotifier notifier(className);
    notifier.setSignalsEnabled(false);
    notifier.setBufferSize(7);

    QSignalSpy spy(&notifier,
                   SIGNAL(changed(QList<TrackerChangeNotifier::Quad>,
                                  QList<TrackerChangeNotifier::Quad>)));

    Receiver receiver;
    QObject::connect(&notifier,
                     SIGNAL(changed(QList<TrackerChangeNotifier::Quad>,
                                    QList<TrackerChangeNotifier::Quad>)),
                     &receiver,
                     SLOT(changed(QList<TrackerChangeNotifier::Quad>,
                                  QList<TrackerChangeNotifier::Quad>)));

    // Do one insert
    insertContacts(QStringList() << "uri1", QStringList() << "Steven1");

    // Lets wait 2 seconds to make sure the signal won't come
    QTime timer;
    timer.start();
    while(timer.elapsed() < 2000)
        QCoreApplication::processEvents();
    QCOMPARE(spy.count(), 0);

    // Do another insert
    insertContacts(QStringList() << "uri2", QStringList() << "Steven2");
    timer.start();
    while(timer.elapsed() < 2000)
        QCoreApplication::processEvents();
    QCOMPARE(spy.count(), 0);

    // Do final insert
    insertContacts(QStringList() << "uri3", QStringList() << "Steven3");
    timer.start();
    while(timer.elapsed() < 5000 && !spy.count())
        QCoreApplication::processEvents();

    QCOMPARE(spy.count(), 1);
    QVERIFY(receiver.deletes.size() == 0);
    QVERIFY(receiver.inserts.size() == 9);

    // Now anoher iteration
    insertContacts(QStringList() << "uri4" << "uri5", QStringList() << "Steven4" << "Steven5");
    timer.start();
    while(timer.elapsed() < 2000)
        QCoreApplication::processEvents();
    QCOMPARE(spy.count(), 1);
    receiver.deletes.clear();
    receiver.inserts.clear();

    insertContacts(QStringList() << "uri6" << "uri7", QStringList() << "Steven6" << "Steven7");
    timer.start();
    while(timer.elapsed() < 5000 && spy.count()!=2)
        QCoreApplication::processEvents();
    QCOMPARE(spy.count(), 2);
    QVERIFY(receiver.deletes.size() == 0);
    QCOMPARE(receiver.inserts.size(), 12);
}

void tst_TrackerSignals::removesBuffered()
{
    //uris and names of our resources
    QStringList uris, names;
    QString urnPattern("unique-uurn-%1-person");
    QString namePattern("Scooby%1");
    for(int i=1; i<11; i++)
    {
        uris << urnPattern.arg(i);
        names << namePattern.arg(i);
    }
    TrackerChangeNotifier notifier(className);
    QSignalSpy spy(&notifier,
                   SIGNAL(changed(QList<TrackerChangeNotifier::Quad>,
                                  QList<TrackerChangeNotifier::Quad>)));

    Receiver receiver;
    QObject::connect(&notifier,
                     SIGNAL(changed(QList<TrackerChangeNotifier::Quad>,
                                    QList<TrackerChangeNotifier::Quad>)),
                     &receiver,
                     SLOT(changed(QList<TrackerChangeNotifier::Quad>,
                                  QList<TrackerChangeNotifier::Quad>)));
    insertContacts(uris, names);
    // Wait until insert is done
    QTime timer;
    timer.start();
    while(timer.elapsed() < 4000 && !spy.count())
        QCoreApplication::processEvents();

    QCOMPARE(receiver.deletes.size(), 0);
    QCOMPARE(receiver.inserts.size(), 30);
    QCOMPARE(spy.count(), 1);

    // clear states
    receiver.deletes.clear();
    receiver.inserts.clear();
    spy.clear();

    notifier.setSignalsEnabled(false);
    notifier.setBufferSize(10);

    // Remove 2 resources, there is 6 signals emitted, so we shoudn't get them
    removeContacts(QStringList() << uris.at(0) << uris.at(1));

    // Lets wait 3 seconds to make sure the signal won't come
    timer.start();
    while(timer.elapsed() < 3000)
        QCoreApplication::processEvents();
    QCOMPARE(receiver.deletes.size(), 0);
    QCOMPARE(receiver.inserts.size(), 0);
    QCOMPARE(spy.count(), 0);

    // Do remove another 2 resources
    removeContacts(QStringList() << uris.at(2) << uris.at(3));

    // Lets wait fir signal
    timer.start();
    while(timer.elapsed() < 5000 && !spy.count())
        QCoreApplication::processEvents();

    QCOMPARE(receiver.deletes.size(), 12);
    QCOMPARE(receiver.inserts.size(), 0);
    QCOMPARE(spy.count(), 1);
    // clear our internal list from resources that don't exist anymore
    uris.removeFirst();
    uris.removeFirst();
    uris.removeFirst();
    uris.removeFirst();

    removeContacts(uris);

    timer.start();
    while(timer.elapsed() < 5000 && spy.count()!=3)
        QCoreApplication::processEvents();

    QCOMPARE(receiver.deletes.size(), 30);
    QCOMPARE(receiver.inserts.size(), 0);
    QCOMPARE(spy.count(), 2);
}

void tst_TrackerSignals::switchBuffer()
{
    TrackerChangeNotifier notifier(className);
    QSignalSpy spy(&notifier,
                   SIGNAL(changed(QList<TrackerChangeNotifier::Quad>,
                                  QList<TrackerChangeNotifier::Quad>)));

    Receiver receiver;
    QObject::connect(&notifier,
                     SIGNAL(changed(QList<TrackerChangeNotifier::Quad>,
                                    QList<TrackerChangeNotifier::Quad>)),
                     &receiver,
                     SLOT(changed(QList<TrackerChangeNotifier::Quad>,
                                  QList<TrackerChangeNotifier::Quad>)));

    // Lets start with buffer inactive. We should get all signals immediately
    insertContacts(QStringList() << "uri1" << "uri2", QStringList() << "Alfredo" << "Mario");
    // Wait until insert is done
    QTime timer;
    timer.start();
    while(timer.elapsed() < 4000 && !spy.count())
        QCoreApplication::processEvents();

    QCOMPARE(receiver.deletes.size(), 0);
    QCOMPARE(receiver.inserts.size(), 6);
    QCOMPARE(spy.count(), 1);
    // Now, one removal
    removeContacts(QStringList() << "uri2");

    timer.start();
    while(timer.elapsed() < 4000 && spy.count()!=2)
        QCoreApplication::processEvents();

    QCOMPARE(receiver.deletes.size(), 3);
    QCOMPARE(receiver.inserts.size(), 6);
    QCOMPARE(spy.count(), 2);

    // Turn buffer on (app going to background)
    notifier.setSignalsEnabled(false);
    notifier.setBufferSize(10);

    // Remove resource, insert 3 new ones
    removeContacts(QStringList() << "uri1");
    insertContacts(QStringList() << "uri5" << "uri6" << "uri7",
                   QStringList() << "Paola" << "Luigi" << "Silvio");

    // Lets wait 3 seconds to make sure the signal won't come
    timer.start();
    while(timer.elapsed() < 3000)
        QCoreApplication::processEvents();

    QCOMPARE(receiver.deletes.size(), 3);
    QCOMPARE(receiver.inserts.size(), 6);
    QCOMPARE(spy.count(), 2);

    // We turn buffer off, like app going to foreground, we should get all pending signals
    notifier.setSignalsEnabled(true);

    // Lets wait for signal
    timer.start();
    while(timer.elapsed() < 5000 && spy.count()!=3)
        QCoreApplication::processEvents();

    QCOMPARE(receiver.deletes.size(), 6);
    QCOMPARE(receiver.inserts.size(), 15);
    QCOMPARE(spy.count(), 3);
}

QTEST_MAIN( tst_TrackerSignals )
#include "tst_tracker_signals.moc"
