/****************************************************************************
**
** Copyright (C) 2010-2011 Nokia Corporation and/or its subsidiary(-ies).
** All rights reserved.
** Contact: Nokia Corporation (ivan.frade@nokia.com)
**
** This file is part of QtSparqlTracker library.
**
** $QT_BEGIN_LICENSE:LGPL$
** GNU Lesser General Public License Usage
** This file may be used under the terms of the GNU Lesser General Public
** License version 2.1 as published by the Free Software Foundation and
** appearing in the file LICENSE.LGPL included in the packaging of this
** file. Please review the following information to ensure the GNU Lesser
** General Public License version 2.1 requirements will be met:
** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** In addition, as a special exception, Nokia gives you certain additional
** rights. These rights are described in the Nokia Qt LGPL Exception
** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU General
** Public License version 3.0 as published by the Free Software Foundation
** and appearing in the file LICENSE.GPL included in the packaging of this
** file. Please review the following information to ensure the GNU General
** Public License version 3.0 requirements will be met:
** http://www.gnu.org/copyleft/gpl.html.
**
** Other Usage
** Alternatively, this file may be used in accordance with the terms and
** conditions contained in a signed written agreement between you and Nokia.
**
** If you have questions regarding the use of this file, please contact
** Nokia at ivan.frade@nokia.com.
** $QT_END_LICENSE$
**
****************************************************************************/

#ifndef LIVETESTHELPERS_H
#define LIVETESTHELPERS_H

#include "modeltesthelpers.h"
#include "TrackerLiveQuery"

#include <QSparqlConnection>
#include <QSparqlQuery>
#include <QSparqlResult>

#include <QAbstractItemModel>
#include <QTest>
#include <QDebug>
#include <QSignalSpy>

// Convenience function for executing a query (normally an insert or a delete),
// waiting for the query to finish, and waiting for Tracker to have sent the
// GraphUpdated signal so that it won't affect the test.
bool executeQuery(QSparqlConnection& conn, const QSparqlQuery &q, bool waitForGraphUpdated = false)
{
    QSparqlResult* r = conn.exec(q);
    r->waitForFinished();
    const bool result = !r->hasError();
    delete r;
    if (waitForGraphUpdated) {
        // Wait for tracker to signal the updates so that they won't be caught
        // by this test case. FIXME: do something more clever. How do we know
        // which classes to watch? We need to wait for multiple GraphUpdated
        // signals.
        QTest::qWait(2000);
    }
    return result;
}

// Like executeQuery above, but uses always a separate Dbus connection
bool executeQuery(const QSparqlQuery &q, bool waitForGraphUpdated = false)
{
    static QSparqlConnection conn("QTRACKER");
    return executeQuery(conn, q, waitForGraphUpdated);
}

bool waitUntilModelFilled(TrackerLiveQuery* lq, int rowCount)
{
    QSignalSpy spy(lq, SIGNAL(initialQueryFinished()));
    // Wait until the initial data is received
    QTime timer;
    timer.start();
    while(timer.elapsed() < 8000 && spy.count() == 0)
        QTest::qWait(100);

    return (spy.count() == 1 && lq->model()->rowCount() == rowCount);
}

#endif
