/****************************************************************************
**
** Copyright (C) 2010-2011 Nokia Corporation and/or its subsidiary(-ies).
** All rights reserved.
** Contact: Nokia Corporation (ivan.frade@nokia.com)
**
** This file is part of QtSparqlTracker library.
**
** $QT_BEGIN_LICENSE:LGPL$
** GNU Lesser General Public License Usage
** This file may be used under the terms of the GNU Lesser General Public
** License version 2.1 as published by the Free Software Foundation and
** appearing in the file LICENSE.LGPL included in the packaging of this
** file. Please review the following information to ensure the GNU Lesser
** General Public License version 2.1 requirements will be met:
** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** In addition, as a special exception, Nokia gives you certain additional
** rights. These rights are described in the Nokia Qt LGPL Exception
** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU General
** Public License version 3.0 as published by the Free Software Foundation
** and appearing in the file LICENSE.GPL included in the packaging of this
** file. Please review the following information to ensure the GNU General
** Public License version 3.0 requirements will be met:
** http://www.gnu.org/copyleft/gpl.html.
**
** Other Usage
** Alternatively, this file may be used in accordance with the terms and
** conditions contained in a signed written agreement between you and Nokia.
**
** If you have questions regarding the use of this file, please contact
** Nokia at ivan.frade@nokia.com.
** $QT_END_LICENSE$
**
****************************************************************************/

#ifndef MODELSIGNALRECORDER_H
#define MODELSIGNALRECORDER_H

#include <QList>
#include <QModelIndex>

class QAbstractItemModel;
class SignalsRecorder;
class QVariant;

class ModelSignalRecorder
{
    QAbstractItemModel* model;
    SignalsRecorder* signalsRecorder;
public:
    ModelSignalRecorder(QAbstractItemModel* model);
    ~ModelSignalRecorder();

    void connectInsertColumnSignals();
    void connectInsertRowSignals();
    void connectRemoveColumnSignals();
    void connectRemoveRowSignals();
    void connectMoveRowSignals();
    void connectDataChangedSignal();

    bool verifyNoSignalsReceived() const;
    bool verifyModelSignalPairReceived
            (const char* preSignal, const char* postSignal,
             const QModelIndex& parent, int start, int end,
             const QModelIndex & destinationParent = QModelIndex(), int destinationRow = -1) const;
    bool verifyInsertColumnSignalPairReceived(const QModelIndex& parent, int start, int end) const;
    bool verifyInsertRowSignalPairReceived(const QModelIndex& parent, int start, int end) const;
    bool verifyRemoveRowSignalPairReceived(const QModelIndex& parent, int start, int end) const;
    bool verifyMoveRowSignalPairReceived(const QModelIndex & sourceParent, int sourceStart, int sourceEnd, const QModelIndex & destinationParent, int destinationRow) const;
    bool verifyDataChangedSignalReceived(const QModelIndex & topLeft, const QModelIndex & bottomRight) const;

    void clearAllSignalData();
};

#endif
