/****************************************************************************
**
** Copyright (C) 2010-2011 Nokia Corporation and/or its subsidiary(-ies).
** All rights reserved.
** Contact: Nokia Corporation (ivan.frade@nokia.com)
**
** This file is part of QtSparqlTracker library.
**
** $QT_BEGIN_LICENSE:LGPL$
** GNU Lesser General Public License Usage
** This file may be used under the terms of the GNU Lesser General Public
** License version 2.1 as published by the Free Software Foundation and
** appearing in the file LICENSE.LGPL included in the packaging of this
** file. Please review the following information to ensure the GNU Lesser
** General Public License version 2.1 requirements will be met:
** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** In addition, as a special exception, Nokia gives you certain additional
** rights. These rights are described in the Nokia Qt LGPL Exception
** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU General
** Public License version 3.0 as published by the Free Software Foundation
** and appearing in the file LICENSE.GPL included in the packaging of this
** file. Please review the following information to ensure the GNU General
** Public License version 3.0 requirements will be met:
** http://www.gnu.org/copyleft/gpl.html.
**
** Other Usage
** Alternatively, this file may be used in accordance with the terms and
** conditions contained in a signed written agreement between you and Nokia.
**
** If you have questions regarding the use of this file, please contact
** Nokia at ivan.frade@nokia.com.
** $QT_END_LICENSE$
**
****************************************************************************/

#include "modeltesthelpers.h"

// Helper functions requiring only QAbstractItemModel (not TrackerLiveQuery).

#include <QAbstractItemModel>
#include <QDebug>
#include <QTest>

Q_DECLARE_METATYPE(QModelIndex)

void equalModels_impl(bool& result, const QAbstractItemModel* lhs, const QAbstractItemModel* rhs)
{
    result = false;

    const int rowCount = lhs->rowCount();
    QCOMPARE( rowCount, rhs->rowCount() );
    const int columnCount = lhs->columnCount();
    QCOMPARE( columnCount, rhs->columnCount() );

    for (int row = 0; row < rowCount; ++row) {
        for (int col = 0; col < columnCount; ++col) {
            QCOMPARE( lhs->data(lhs->index(row,col)), rhs->data(rhs->index(row,col)) );
        }
    }

    result = true;
}

bool equalModels(const QAbstractItemModel* lhs, const QAbstractItemModel* rhs)
{
    bool result = false;
    equalModels_impl(result, lhs, rhs);
    return result;
}

void printModel(const QAbstractItemModel* model)
{
    qDebug() << "printModel:";
    for (int r = 0; r < model->rowCount(); ++r)
        for (int c = 0; c < model->columnCount(); ++c)
            qDebug() << "(" << r << "," << c << ")" <<
                model->data(model->index(r, c)).toString();
}

QStringList getRows(QAbstractItemModel* model, QList<int> columns)
{
    QStringList data;
    for (int i = 0; i < model->rowCount(); ++i) {
        QString d;
        Q_FOREACH(int c, columns)
            d.append(model->data(model->index(i, c)).toString() + ":");
        data << d;
    }
    return data;
}

ModelRowData rowData(const QAbstractItemModel* model, int row)
{
    ModelRowData data;
    for (int col = 0; col < model->columnCount(); ++col) {
        data << model->data(model->index(row, col));
    }
    return data;
}

void equalRowData_impl(bool& result, const QAbstractItemModel* model, int row, const ModelRowData& expectedData)
{
    result = false;
    QCOMPARE( model->columnCount(), expectedData.count() );
    for (int col = 0; col < model->columnCount(); ++col) {
        QCOMPARE( model->data(model->index(row, col)), expectedData[col] );
    }
    result = true;
}

bool equalRowData(const QAbstractItemModel* model, int row, const ModelRowData& expectedData)
{
    bool result = false;
    equalRowData_impl(result, model, row, expectedData);
    return result;
}

void verifySignalParameters_impl(bool& result, const QList<QVariant>& params, const QModelIndex& parent, int start, int end,
                                 const QModelIndex & destinationParent, int destinationRow)
{
    result = false;
    qRegisterMetaType<QModelIndex>("QModelIndex");

    QVERIFY( params.count() == 3 || params.count() == 5 );

    QCOMPARE( params[0].value<QModelIndex>(), parent );
    QCOMPARE( params[1].value<int>(), start );
    QCOMPARE( params[2].value<int>(), end );

    if (params.count() == 3) {
        QCOMPARE( destinationParent, QModelIndex() );
        QCOMPARE( destinationRow, -1 );
    }
    else {
        QCOMPARE( params[3].value<QModelIndex>(), destinationParent );
        QCOMPARE( params[4].value<int>(), destinationRow );
    }
    result = true;
}

bool verifySignalParameters(const QList<QVariant>& params, const QModelIndex& parent, int start, int end,
                            const QModelIndex & destinationParent, int destinationRow)
{
    bool result = false;
    verifySignalParameters_impl(result, params, parent, start, end, destinationParent, destinationRow);
    return result;
}
