/****************************************************************************
**
** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
** All rights reserved.
** Contact: Nokia Corporation (ivan.frade@nokia.com)
**
** This file is part of QtSparqlTracker library.
**
** $QT_BEGIN_LICENSE:LGPL$
** GNU Lesser General Public License Usage
** This file may be used under the terms of the GNU Lesser General Public
** License version 2.1 as published by the Free Software Foundation and
** appearing in the file LICENSE.LGPL included in the packaging of this
** file. Please review the following information to ensure the GNU Lesser
** General Public License version 2.1 requirements will be met:
** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** In addition, as a special exception, Nokia gives you certain additional
** rights. These rights are described in the Nokia Qt LGPL Exception
** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU General
** Public License version 3.0 as published by the Free Software Foundation
** and appearing in the file LICENSE.GPL included in the packaging of this
** file. Please review the following information to ensure the GNU General
** Public License version 3.0 requirements will be met:
** http://www.gnu.org/copyleft/gpl.html.
**
** Other Usage
** Alternatively, this file may be used in accordance with the terms and
** conditions contained in a signed written agreement between you and Nokia.
**
** If you have questions regarding the use of this file, please contact
** Nokia at ivan.frade@nokia.com.
** $QT_END_LICENSE$
**
****************************************************************************/

#ifndef SIGNALSRECORDER_H
#define SIGNALSRECORDER_H

#include <QObject>
#include <QList>
#include <QSet>
#include <QSignalSpy>
#include <QTimer>

class SignalList
{
public:
	SignalList();
	SignalList& append(QObject* object, const char* signal, int numExpected=1);
	bool isEmpty() const;
	int indexOf(QObject* object, const QByteArray& signal) const;
	void remove(QObject* object, const QByteArray& signal);

	struct SignalDesc
	{
		QObject* object;
		QByteArray originalSignal;
		QByteArray signal;
		int numExpected;
	};

	QList<SignalDesc> asList() const;

private:
	QList<SignalDesc> signalDescs_;
};

class SignalsRecorder : public QObject
{
    Q_OBJECT
public:
	class SignalData
	{
	public:
		SignalData(QObject* object, const QByteArray& signal, const QList<QVariant>& params, int seqId);
		QList<QVariant> params() const { return params_; }
		int             seqId()  const { return seqId_;  }
		bool hasObject(QObject* object) const;
		bool hasSignal(const char* signal) const;
		bool matches(QObject* object, const char* signal) const;
		QByteArray signal() const;

	private:
		friend bool operator==(const SignalsRecorder::SignalData&, const SignalsRecorder::SignalData&);
		friend bool operator!=(const SignalsRecorder::SignalData&, const SignalsRecorder::SignalData&);
		friend QDebug operator<<(QDebug, const SignalsRecorder::SignalData&);
		QObject* object_;
		QByteArray signal_;
		QList<QVariant> params_;
		int seqId_;
	};

    explicit SignalsRecorder(QObject *parent = 0);
	~SignalsRecorder();

	bool listen(QObject* object, const char* signal);
	bool listen(const SignalList& signalList);
	bool waitForSignals(const SignalList& expectedSignals, int timeoutMsec);
	QList<SignalData> allSignalData() const;
	QList<SignalData> signalData(QObject* object, const char* signal) const;
	QList<SignalData> signalData(const SignalList& signalList) const;
	void clearAllSignalData();
	void clearSignalData(QObject* object, const char* signal);
	void clearSignalData(const SignalList& signalList);

signals:

public slots:

private:
	int signalReceived(QObject* object, const QByteArray& signal);
	QList<SignalData> signalData(QObject* object, const QByteArray& signal) const;
	void clearSignalData(QObject* object, const QByteArray& signal);

private:
	class SignalsRecorderSpy;
	friend class SignalsRecorderSpy;
	QList<SignalsRecorderSpy*> signalSpies_;
	int nextSeqId_;
	SignalList expectedSignals_;
	QTimer timeoutTimer_;
};

bool operator==(const SignalsRecorder::SignalData& lhs, const SignalsRecorder::SignalData& rhs);
bool sequenceOrdering(const SignalsRecorder::SignalData& lhs, const SignalsRecorder::SignalData& rhs);

#endif // SIGNALSRECORDER_H
