/****************************************************************************
**
** Copyright (C) 2010-2011 Nokia Corporation and/or its subsidiary(-ies).
** All rights reserved.
** Contact: Nokia Corporation (ivan.frade@nokia.com)
**
** This file is part of QtSparqlTracker library.
**
** $QT_BEGIN_LICENSE:LGPL$
** Commercial Usage
** Licensees holding valid Qt Commercial licenses may use this file in
** accordance with the Qt Commercial License Agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and Nokia.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 2.1 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU Lesser General Public License version 2.1 requirements
** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** In addition, as a special exception, Nokia gives you certain additional
** rights.  These rights are described in the Nokia Qt LGPL Exception
** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3.0 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU General Public License version 3.0 requirements will be
** met: http://www.gnu.org/copyleft/gpl.html.
**
** If you have questions regarding the use of this file, please contact
** Nokia at ivan.frade@nokia.com.
** $QT_END_LICENSE$
**
****************************************************************************/

#include "TrackerChangeNotifier"

#include <QtSparql>

#include <QDebug>
#include <QCoreApplication>

class Receiver : public QObject
{
    Q_OBJECT

public Q_SLOTS:
    void changed(QList<TrackerChangeNotifier::Quad> d, QList<TrackerChangeNotifier::Quad> i)
    {
        qDebug() << "Signal arrived at" << QDateTime::currentDateTime();
        TrackerChangeNotifier* tcn = qobject_cast<TrackerChangeNotifier*>(sender());
        Q_ASSERT(tcn != 0);
        qDebug() << "Class:" << tcn->watchedClass();
        qDebug() << "Deletes:" << d;
        qDebug() << "Inserts:" << i;
    }
};

int main(int argc, char *argv[])
{
    if (argc < 2) {
        qDebug() << "Usage:" << argv[0] << "CLASS..";
        return 1;
    }

    QCoreApplication app(argc, argv);

    QStringList shortClasses;
    QString querySnippet;


    for (int i = 1; i < argc; ++i) {
        shortClasses << QString(argv[i]);
        querySnippet.append(QString("%1 ").arg(argv[i]));
    }

    Receiver receiver;

    QSparqlConnection conn("QTRACKER_DIRECT");
    QSparqlQuery q(QString("SELECT %1 WHERE { }").arg(querySnippet));
    QSparqlResult* r = conn.exec(q);
    r->waitForFinished();
    if (r->next()) {
        for (int i = 0; i < shortClasses.size(); ++i) {
            QString longClass = r->value(i).toString();
            TrackerChangeNotifier* tcn = new TrackerChangeNotifier(longClass);
            QObject::connect(tcn,
                             SIGNAL(changed(QList<TrackerChangeNotifier::Quad>,
                                            QList<TrackerChangeNotifier::Quad>)),
                             &receiver,
                             SLOT(changed(QList<TrackerChangeNotifier::Quad>,
                                          QList<TrackerChangeNotifier::Quad>)));
        }
    }
    delete r;
    return app.exec();
}

#include "main.moc"
